@@ -5,7 +5,6 @@ import logging
 logger = logging.getLogger(__name__)
 
 import synapse.exc as s_exc
-import synapse.glob as s_glob
 import synapse.common as s_common
 import synapse.telepath as s_telepath
 
@@ -122,8 +121,8 @@ dmonwrap = (
 
 async def t2call(link, meth, args, kwargs):
     '''
-    Call the given meth(*args, **kwargs) and handle the response
-    to provide telepath task v2 events to the given link.
+    Call the given ``meth(*args, **kwargs)`` and handle the response to provide
+    telepath task v2 events to the given link.
     '''
     try:
 
@@ -136,7 +135,9 @@ async def t2call(link, meth, args, kwargs):
 
             first = True
             if isinstance(valu, types.AsyncGeneratorType):
+
                 async for item in valu:
+
                     if first:
                         await link.tx(('t2:genr', {}))
                         first = False
@@ -271,7 +272,11 @@ class Daemon(s_base.Base):
 
             sslctx = None
             if scheme == 'ssl':
-                sslctx = self.certdir.getServerSSLContext(hostname=host)
+                hostname = None
+                query = info.get('query')
+                if query is not None:
+                    hostname = query.get('hostname', host)
+                sslctx = self.certdir.getServerSSLContext(hostname=hostname)
 
             server = await s_link.listen(host, port, self._onLinkInit, ssl=sslctx)
 
@@ -312,10 +317,6 @@ class Daemon(s_base.Base):
             except Exception as e:  # pragma: no cover
                 logger.warning('Error during socket server close()', exc_info=e)
 
-        for name, share in self.shared.items():
-            if isinstance(share, s_base.Base):
-                await share.fini()
-
         finis = [sess.fini() for sess in self.sessions.values()]
         if finis:
             await asyncio.wait(finis)
@@ -324,6 +325,10 @@ class Daemon(s_base.Base):
         if finis:
             await asyncio.wait(finis)
 
+        for _, share in self.shared.items():
+            if isinstance(share, s_base.Base):
+                await share.fini()
+
     async def _onLinkInit(self, link):
 
         async def rxloop():
@@ -353,6 +358,9 @@ class Daemon(s_base.Base):
         except asyncio.CancelledError: # pragma: no cover
             raise
 
+        except ConnectionResetError:
+            logger.debug('Dmon.onLinkMesg Handler: connection reset')
+
         except Exception:
             logger.exception('Dmon.onLinkMesg Handler: %.80r' % (mesg,))
 
@@ -401,6 +409,7 @@ class Daemon(s_base.Base):
                 raise s_exc.NoSuchName(name=name)
 
             sess = await Sess.anit()
+
             async def sessfini():
                 self.sessions.pop(sess.iden, None)
 
