@@ -5,6 +5,7 @@ import asyncio
 import logging
 import calendar
 import datetime
+import functools
 import itertools
 from datetime import timezone as tz
 from collections.abc import Iterable, Mapping
@@ -13,12 +14,60 @@ import synapse.exc as s_exc
 import synapse.common as s_common
 
 import synapse.lib.base as s_base
+import synapse.lib.config as s_config
 import synapse.lib.provenance as s_provenance
 
 # Agenda: manages running one-shot and periodic tasks in the future ("appointments")
 
 logger = logging.getLogger(__name__)
 
+reqValidCdef = s_config.getJsValidator({
+    'type': 'object',
+    'properties': {
+        'storm': {'type': 'string'},
+        'creator': {'type': 'string', 'pattern': s_config.re_iden},
+        'incunit': {
+            'oneOf': [
+                {'type': 'null'},
+                {'enum': ['year', 'month', 'dayofmonth', 'dayofweek', 'day', 'hour', 'minute']}
+            ]
+        },
+        'incvals': {
+            'type': ['array', 'number', 'null'],
+            'items': {'type': 'number'}
+        },
+        'reqs': {
+            'oneOf': [
+                {
+                    '$ref': '#/definitions/req',
+                },
+                {
+                    'type': ['array'],
+                    'items': {'$ref': '#/definitions/req'},
+                },
+            ]
+        },
+    },
+    'additionalProperties': False,
+    'required': ['creator', 'storm'],
+    'dependencices': {
+        'incvals': ['incunit'],
+        'incunit': ['incvals'],
+    },
+    'definitions': {
+        'req': {
+            'type': 'object',
+            'properties': {
+                'minute': {'type': 'number'},
+                'hour': {'type': 'number'},
+                'dayofmonth': {'type': 'number'},
+                'month': {'type': 'number'},
+                'year': {'type': 'number'},
+            }
+        }
+    }
+})
+
 def _dayofmonth(hardday, month, year):
     '''
     Returns a valid day of the month given the desired value.
@@ -86,7 +135,8 @@ class ApptRec:
     Represents a single element of a single combination of an appointment
     '''
     def __init__(self, reqdict, incunit=None, incval=1):
-        self.reqdict = reqdict
+        if incunit is not None:
+            incunit = TimeUnit(incunit)
         self.incunit = incunit
         self.incval = incval if incunit is not None else None
 
@@ -110,9 +160,10 @@ class ApptRec:
             if not boundmin <= incval <= boundmax:
                 raise s_exc.BadTime(mesg='Out of bounds incval')
 
+        reqdict = {TimeUnit(k): v for k, v in reqdict.items()}
+        self.reqdict = reqdict
+
         for reqkey, reqval in reqdict.items():
-            if reqkey not in TimeUnit:
-                raise s_exc.BadTime(mesg='Keys of reqdict parameter must be valid TimeUnit values')
             boundmin, boundmax = _UnitBounds[reqkey][0]
             if not boundmin <= reqval <= boundmax:
                 raise s_exc.BadTime(mesg='Out of bounds reqdict value')
@@ -249,7 +300,7 @@ class _Appt:
     Each such entry has a list of ApptRecs.  Each time the appointment is scheduled, the nexttime of the appointment is
     the lowest nexttime of all its ApptRecs.
     '''
-    def __init__(self, stor, iden, recur, indx, query, useriden, recs, nexttime=None):
+    def __init__(self, stor, iden, recur, indx, query, creator, recs, nexttime=None):
         self.doc = ''
         self.name = ''
         self.stor = stor
@@ -257,7 +308,7 @@ class _Appt:
         self.recur = recur # does this appointment repeat
         self.indx = indx  # incremented for each appt added ever.  Used for nexttime tiebreaking for stable ordering
         self.query = query  # query to run
-        self.useriden = useriden # user iden to run query as
+        self.creator = creator # user iden to run query as
         self.recs = recs  # List[ApptRec]  list of the individual entries to calculate next time from
         self._recidxnexttime = None # index of rec who is up next
 
@@ -279,14 +330,26 @@ class _Appt:
         self.lastresult = None
         self.enabled = True
 
-    def getRuntInfo(self):
-        buid = s_common.buid(('syn:cron', self.iden))
-        return buid, (
-            ('*syn:cron', self.iden),
-            ('doc', self.doc),
-            ('name', self.name),
-            ('storm', self.query),
-        )
+    def getStorNode(self, form):
+        ndef = (form.name, form.type.norm(self.iden)[0])
+        buid = s_common.buid(ndef)
+
+        props = {
+            'doc': self.doc,
+            'name': self.name,
+            'storm': self.query,
+        }
+
+        pnorms = {}
+        for prop, valu in props.items():
+            formprop = form.props.get(prop)
+            if formprop is not None and valu is not None:
+                pnorms[prop] = formprop.type.norm(valu)[0]
+
+        return (buid, {
+            'ndef': ndef,
+            'props': pnorms
+        })
 
     def __eq__(self, other):
         ''' For heap logic to sort upcoming events lower '''
@@ -306,7 +369,7 @@ class _Appt:
             'iden': self.iden,
             'indx': self.indx,
             'query': self.query,
-            'useriden': self.useriden,
+            'creator': self.creator,
             'recs': [d.pack() for d in self.recs],
             'nexttime': self.nexttime,
             'startcount': self.startcount,
@@ -321,7 +384,7 @@ class _Appt:
         if val['ver'] != 1:
             raise s_exc.BadStorageVersion(mesg=f"Found version {val['ver']}")  # pragma: no cover
         recs = [ApptRec.unpack(tupl) for tupl in val['recs']]
-        appt = cls(stor, val['iden'], val['recur'], val['indx'], val['query'], val['useriden'], recs, val['nexttime'])
+        appt = cls(stor, val['iden'], val['recur'], val['indx'], val['query'], val['creator'], recs, val['nexttime'])
         appt.doc = val.get('doc', '')
         appt.name = val.get('name', '')
         appt.startcount = val['startcount']
@@ -370,17 +433,6 @@ class _Appt:
             self.nexttime = None
             return
 
-    async def reqAllowed(self, user, perm):
-        if not await self.allowed(user, perm):
-            mesg = 'User does not own or have permissions to cron job.'
-            raise s_exc.AuthDeny(perm=perm, mesg=mesg)
-
-    async def allowed(self, user, perm):
-        if user.iden == self.useriden:
-            return True
-
-        return user.allowed(perm)
-
     async def setDoc(self, text):
         '''
         Set the doc field of an appointment.
@@ -414,26 +466,14 @@ class Agenda(s_base.Base):
 
         self._hivedict = await self.core.hive.dict(('agenda', 'appts'))  # Persistent storage
         self.onfini(self._hivedict)
+        self.onfini(self.stop)
 
         self.enabled = False
         self._schedtask = None  # The task of the scheduler loop.  Doesn't run until we're enabled
+
+        self._running_tasks = []  # The actively running cron job tasks
         await self._load_all()
 
-    async def onLiftRunts(self, full, valu=None, cmpr=None):
-
-        if valu is None:
-            for iden, cjob in self.appts.items():
-                yield cjob.getRuntInfo()
-
-            return
-
-        iden = str(valu)
-        cjob = self.appts.get(iden)
-        if cjob is None:
-            return
-
-        yield cjob.getRuntInfo()
-
     async def start(self):
         '''
         Enable cron jobs to start running, start the scheduler loop
@@ -454,6 +494,16 @@ class Agenda(s_base.Base):
         self._schedtask = self.schedCoro(self._scheduleLoop())
         self.enabled = True
 
+    async def stop(self):
+        "Cancel the scheduler loop, and set self.enabled to False."
+        if not self.enabled:
+            return
+        self._schedtask.cancel()
+        for task in self._running_tasks:
+            task.cancel()
+
+        self.enabled = False
+
     async def _load_all(self):
         '''
         Load all the appointments from persistent storage
@@ -516,7 +566,7 @@ class Agenda(s_base.Base):
     def list(self):
         return list(self.appts.items())
 
-    async def add(self, useriden, query: str, reqs, incunit=None, incvals=None, doc=''):
+    async def add(self, cdef):
         '''
         Persistently adds an appointment
 
@@ -540,13 +590,19 @@ class Agenda(s_base.Base):
         Returns:
             iden of new appointment
         '''
-        iden = s_common.guid()
+        iden = cdef['iden']
+        incunit = cdef.get('incunit')
+        incvals = cdef.get('incvals')
+        reqs = cdef.get('reqs', {})
+        query = cdef.get('storm')
+        creator = cdef.get('creator')
+
         recur = incunit is not None
         indx = self._next_indx
         self._next_indx += 1
 
-        if reqs is None:
-            reqs = {}
+        if iden in self.appts:
+            raise s_exc.DupIden()
 
         if not query:
             raise ValueError('empty query')
@@ -561,7 +617,7 @@ class Agenda(s_base.Base):
             reqs = [reqs]
 
         # Find all combinations of values in reqdict values and incvals values
-        recs = []
+        recs = []  # type: ignore
         for req in reqs:
 
             reqdicts = self._dictproduct(req)
@@ -569,14 +625,14 @@ class Agenda(s_base.Base):
                 incvals = (incvals, )
             recs.extend(ApptRec(rd, incunit, v) for (rd, v) in itertools.product(reqdicts, incvals))
 
-        appt = _Appt(self, iden, recur, indx, query, useriden, recs)
+        appt = _Appt(self, iden, recur, indx, query, creator, recs)
         self._addappt(iden, appt)
 
-        appt.doc = doc
+        appt.doc = cdef.get('doc', '')
 
         await self._storeAppt(appt)
 
-        return iden
+        return appt.pack()
 
     async def get(self, iden):
 
@@ -662,12 +718,16 @@ class Agenda(s_base.Base):
 
             now = time.time()
             while self.apptheap and self.apptheap[0].nexttime <= now:
+
                 appt = heapq.heappop(self.apptheap)
                 appt.updateNexttime(now)
+
                 if appt.nexttime:
                     heapq.heappush(self.apptheap, appt)
-                if not appt.enabled:
+
+                if not appt.enabled or not self.enabled:
                     continue
+
                 if appt.isrunning:
                     logger.warning(
                         'Appointment %s is still running from previous time when scheduled to run.  Skipping.',
@@ -679,15 +739,16 @@ class Agenda(s_base.Base):
         '''
         Fire off the task to make the storm query
         '''
-        user = self.core.auth.user(appt.useriden)
+        user = self.core.auth.user(appt.creator)
         if user is None:
-            logger.warning('Unknown user %s in stored appointment', appt.useriden)
+            logger.warning('Unknown user %s in stored appointment', appt.creator)
             await self._markfailed(appt)
             return
-        await self.core.boss.execute(self._runJob(user, appt), f'Cron {appt.iden}', user,
-                                     info={'iden': appt.iden,
-                                           'query': appt.query,
-                                           })
+        info = {'iden': appt.iden, 'query': appt.query}
+        task = await self.core.boss.execute(self._runJob(user, appt), f'Cron {appt.iden}', user, info=info)
+        self._running_tasks.append(task)
+
+        task.onfini(functools.partial(self._running_tasks.remove, task))
 
     async def _markfailed(self, appt):
         appt.lastfinishtime = appt.laststarttime = time.time()
@@ -711,7 +772,8 @@ class Agenda(s_base.Base):
             logger.info('Agenda executing for iden=%s, user=%s, query={%s}', appt.iden, user.name, appt.query)
             starttime = time.time()
             try:
-                async for _ in self.core.eval(appt.query, user=user):  # NOQA
+                opts = {'user': user.iden}
+                async for node in self.core.eval(appt.query, opts=opts):
                     count += 1
             except asyncio.CancelledError:
                 result = 'cancelled'
