@@ -1,4 +1,5 @@
 import gc
+import os
 import atexit
 import signal
 import asyncio
@@ -18,6 +19,8 @@ import synapse.lib.coro as s_coro
 
 logger = logging.getLogger(__name__)
 
+OMIT_FINI_WARNS = os.environ.get('SYNDEV_OMIT_FINI_WARNS', False)
+
 def _fini_atexit(): # pragma: no cover
 
     for item in gc.get_objects():
@@ -31,7 +34,7 @@ def _fini_atexit(): # pragma: no cover
         if item.isfini:
             continue
 
-        if not item._fini_atexit:
+        if not item._fini_atexit and not OMIT_FINI_WARNS:
             if __debug__:
                 print(f'At exit: Missing fini for {item}')
                 for depth, call in enumerate(item.call_stack[:-2]):
@@ -94,12 +97,18 @@ class Base:
             await self.__anit__(*args, **kwargs)
 
         except Exception:
-
             if self.anitted:
                 await self.fini()
 
             raise
 
+        try:
+            await self.postAnit()
+        except Exception:
+            logger.exception('Error during postAnit callback.')
+            await self.fini()
+            raise
+
         return self
 
     async def __anit__(self):
@@ -138,8 +147,20 @@ class Base:
         self._fini_atexit = False
         self._active_tasks = set()  # the free running tasks associated with me
 
-    async def enter_context(self, item):
+    async def postAnit(self):
+        '''
+        Method called after self.__anit__() has completed, but before anit() returns the object to the caller.
+        '''
+        pass
 
+    async def enter_context(self, item):
+        '''
+        Modeled on Python's contextlib.ExitStack.enter_context.  Enters a new context manager and adds its __exit__()
+        and __aexit__ method to its onfini handlers.
+
+        Returns:
+            The result of item’s own __aenter__ or __enter__() method.
+        '''
         async def fini():
             meth = getattr(item, '__aexit__', None)
             if meth is not None:
@@ -159,11 +180,12 @@ class Base:
             return await entr()
 
         entr = getattr(item, '__enter__', None)
-        if entr is not None:
-            async def fini():
-                item.__exit__(None, None, None)
-            self.onfini(fini)
-            return entr()
+        assert entr is not None
+
+        async def fini():
+            item.__exit__(None, None, None)
+        self.onfini(fini)
+        return entr()
 
     def onfini(self, func):
         '''
@@ -383,11 +405,6 @@ class Base:
 
         self.isfini = True
 
-        fevt = self.finievt
-
-        if fevt is not None:
-            fevt.set()
-
         for base in list(self.tofini):
             await base.fini()
 
@@ -406,6 +423,12 @@ class Base:
 
         self._syn_funcs.clear()
         self._fini_funcs.clear()
+
+        fevt = self.finievt
+
+        if fevt is not None:
+            fevt.set()
+
         return 0
 
     @contextlib.contextmanager
@@ -475,11 +498,12 @@ class Base:
         def taskDone(task):
             self._active_tasks.remove(task)
             try:
-                task.result()
+                if not task.done():
+                    task.result()
             except asyncio.CancelledError:
                 pass
             except Exception:
-                logger.exception('Task scheduled through Base.schedCoro raised exception')
+                logger.exception('Task %s scheduled through Base.schedCoro raised exception', task)
 
         self._active_tasks.add(task)
         task.add_done_callback(taskDone)
@@ -498,7 +522,7 @@ class Base:
             **kwargs:
 
         Notes:
-            This method may called from outside of the event loop on a different thread.
+            This method may be called from outside of the event loop on a different thread.
 
         Returns:
             concurrent.futures.Future: A Future representing the eventual function execution.
@@ -735,6 +759,44 @@ class BaseRef(Base):
         # change during iteration exceptions
         return iter(list(self.base_by_name.values()))
 
+async def schedGenr(genr, maxsize=100):
+    '''
+    Schedule a generator to run on a separate task and yield results to this task (pipelined generator).
+    '''
+    q = asyncio.Queue(maxsize=maxsize)
+
+    async def genrtask(base):
+        try:
+            async for item in genr:
+                await q.put((True, item))
+
+            await q.put((False, None))
+
+        except asyncio.CancelledError:
+            raise
+
+        except Exception:
+            if not base.isfini:
+                await q.put((False, None))
+            raise
+
+    async with await Base.anit() as base:
+
+        task = base.schedCoro(genrtask(base))
+
+        while not base.isfini:
+
+            ok, retn = await q.get()
+
+            if ok:
+                yield retn
+                # since we are a pipeline, yield every time...
+                await asyncio.sleep(0)
+                continue
+
+            await task
+            return
+
 async def main(coro): # pragma: no cover
     base = await coro
     if isinstance(base, Base):
