@@ -658,7 +658,7 @@ class CertDir:
 
         fname = os.path.split(path)[1]
         parts = fname.rsplit('.', 1)
-        ext = parts[1] if len(parts) is 2 else None
+        ext = parts[1] if len(parts) == 2 else None
 
         if not ext or ext not in ('crt', 'key', 'p12'):
             mesg = 'importFile only supports .crt, .key, .p12 extensions'
@@ -857,12 +857,16 @@ class CertDir:
         sslctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
         if hostname is None:
             hostname = socket.gethostname()
+
         certfile = self.getHostCertPath(hostname)
         if certfile is None:
-            raise s_exc.NoCertKey('Missing .crt for %s' % hostname)
+            mesg = f'Missing TLS certificate file for host: {hostname}'
+            raise s_exc.NoCertKey(mesg=mesg)
+
         keyfile = self.getHostKeyPath(hostname)
         if keyfile is None:
-            raise s_exc.NoCertKey('Missing .key for %s' % hostname)
+            mesg = f'Missing TLS key file for host: {hostname}'
+            raise s_exc.NoCertKey(mesg=mesg)
 
         sslctx.load_cert_chain(certfile, keyfile)
 
