@@ -1,3 +1,7 @@
+'''
+Spawn is mechanism so that a cortex can execute different queries in separate processes
+'''
+
 import os
 import asyncio
 import logging
@@ -11,6 +15,7 @@ import concurrent.futures
 import synapse.exc as s_exc
 import synapse.glob as s_glob
 import synapse.common as s_common
+import synapse.cortex as s_cortex
 import synapse.daemon as s_daemon
 import synapse.telepath as s_telepath
 import synapse.datamodel as s_datamodel
@@ -21,10 +26,9 @@ import synapse.lib.coro as s_coro
 import synapse.lib.hive as s_hive
 import synapse.lib.link as s_link
 import synapse.lib.view as s_view
-
 import synapse.lib.storm as s_storm
 import synapse.lib.dyndeps as s_dyndeps
-import synapse.lib.grammar as s_grammar
+import synapse.lib.hiveauth as s_hiveauth
 
 logger = logging.getLogger(__name__)
 
@@ -40,6 +44,9 @@ async def storm(core, item):
     opts = storminfo.get('opts')
     text = storminfo.get('query')
 
+    if opts is None:
+        opts = {}
+
     user = core.auth.user(useriden)
     if user is None:
         raise s_exc.NoSuchUser(iden=useriden)
@@ -48,7 +55,8 @@ async def storm(core, item):
     if view is None:
         raise s_exc.NoSuchView(iden=viewiden)
 
-    async for mesg in view.streamstorm(text, opts=opts, user=user):
+    opts['user'] = useriden
+    async for mesg in view.storm(text, opts=opts):
         yield mesg
 
 async def _innerloop(core, todo, done):
@@ -93,7 +101,10 @@ async def _workloop(spawninfo, todo, done):
     '''
     s_glob.iAmLoop()
 
-    async with await SpawnCore.anit(spawninfo) as core:
+    ctorname = spawninfo.get('spawncorector')
+    ctor = s_dyndeps.tryDynLocal(ctorname)
+
+    async with await ctor.anit(spawninfo) as core:
 
         while not core.isfini:
 
@@ -201,7 +212,12 @@ class SpawnProc(s_base.Base):
 
         def doit():
             self.todo.put(mesg)
-            return self.done.get()
+            try:
+                return self.done.get()
+            except (TypeError, OSError) as e:
+                logger.warning('Queue torn out from underneath me. (%s)', e)
+                assert self.isfini
+                return True
 
         return await self.executor(doit)
 
@@ -290,6 +306,9 @@ class SpawnPool(s_base.Base):
         return proc
 
 class SpawnCore(s_base.Base):
+    '''
+    A SpawnCore instance is the substitute for a Cortex in non-cortex processes
+    '''
 
     async def __anit__(self, spawninfo):
 
@@ -298,56 +317,60 @@ class SpawnCore(s_base.Base):
         self.pid = os.getpid()
         self.views = {}
         self.layers = {}
+        self.nexsroot = None
+        self.isleader = False
         self.spawninfo = spawninfo
 
         self.conf = spawninfo.get('conf')
         self.iden = spawninfo.get('iden')
         self.dirn = spawninfo.get('dirn')
 
+        self.trigson = self.conf.get('trigger:enable')
+
+        self.svcsbyiden = {}
+        self.svcsbyname = {}
+
         self.stormcmds = {}
+        self.storm_cmd_ctors = {}
+        self.storm_cmd_cdefs = {}
         self.stormmods = spawninfo['storm']['mods']
+        self.pkginfo = spawninfo['storm']['pkgs']
+        self.svcinfo = spawninfo['storm']['svcs']
+
+        self.model = s_datamodel.Model()
+        self.model.addDataModels(spawninfo.get('model'))
+
+        self.stormpkgs = {}     # name: pkgdef
+        await self._initStormCmds()
+
+        for sdef in self.svcinfo:
+            await self._addStormSvc(sdef)
+
+        for pkgdef in self.pkginfo:
+            await self._tryLoadStormPkg(pkgdef)
 
         for name, ctor in spawninfo['storm']['cmds']['ctors']:
             self.stormcmds[name] = ctor
 
         for name, cdef in spawninfo['storm']['cmds']['cdefs']:
-            ctor = functools.partial(s_storm.PureCmd, cdef)
-            self.stormcmds[name] = ctor
+            self.storm_cmd_cdefs[name] = cdef
 
         self.libroot = spawninfo.get('storm').get('libs')
 
         self.boss = await s_boss.Boss.anit()
         self.onfini(self.boss.fini)
 
-        self.model = s_datamodel.Model()
-        self.model.addDataModels(spawninfo.get('model'))
-
         self.prox = await s_telepath.openurl(f'cell://{self.dirn}')
         self.onfini(self.prox.fini)
 
         self.hive = await s_hive.openurl(f'cell://{self.dirn}', name='*/hive')
         self.onfini(self.hive)
 
-        # TODO cortex configured for remote auth...
         node = await self.hive.open(('auth',))
-        self.auth = await s_hive.HiveAuth.anit(node)
+        self.auth = await s_hiveauth.Auth.anit(node)
         self.onfini(self.auth.fini)
-
         for layrinfo in self.spawninfo.get('layers'):
-
-            iden = layrinfo.get('iden')
-            path = ('cortex', 'layers', iden)
-
-            ctorname = layrinfo.get('ctor')
-
-            ctor = s_dyndeps.tryDynLocal(ctorname)
-
-            node = await self.hive.open(path)
-            layr = await ctor(self, node, readonly=True)
-
-            self.onfini(layr)
-
-            self.layers[iden] = layr
+            await self._initLayr(layrinfo)
 
         for viewinfo in self.spawninfo.get('views'):
 
@@ -361,36 +384,48 @@ class SpawnCore(s_base.Base):
 
             self.views[iden] = view
 
-        # initialize pass-through methods from the telepath proxy
-        # Lift
-        self.runRuntLift = self.prox.runRuntLift
-        # StormType Queue APIs
-        self.addCoreQueue = self.prox.addCoreQueue
-        self.hasCoreQueue = self.prox.hasCoreQueue
-        self.delCoreQueue = self.prox.delCoreQueue
-        self.getCoreQueue = self.prox.getCoreQueue
-        self.getCoreQueues = self.prox.getCoreQueues
-        self.getsCoreQueue = self.prox.getsCoreQueue
-        self.putCoreQueue = self.prox.putCoreQueue
-        self.putsCoreQueue = self.prox.putsCoreQueue
-        self.cullCoreQueue = self.prox.cullCoreQueue
-        # Feedfunc support
-        self.getFeedFuncs = self.prox.getFeedFuncs
-        # storm pkgfuncs
-        self.addStormPkg = self.prox.addStormPkg
-        self.delStormPkg = self.prox.delStormPkg
-        self.getStormPkgs = self.prox.getStormPkgs
+        for view in self.views.values():
+            view.init2()
 
-        # TODO: Add Dmon management functions ($lib.dmon support)
-        # TODO: Add Axon management functions ($lib.bytes support)
+        #TODO maybe we should subclass telepath client
+        self.addStormDmon = self.prox.addStormDmon
+        self.delStormDmon = self.prox.delStormDmon
+        self.getStormDmon = self.prox.getStormDmon
+        self.getStormDmons = self.prox.getStormDmons
 
-    def getStormQuery(self, text):
-        '''
-        Parse storm query text and return a Query object.
-        '''
-        query = s_grammar.Parser(text).query()
-        query.init(self)
-        return query
+        # Cell specific apis
+        self.setHiveKey = self.prox.setHiveKey
+        self.getHiveKey = self.prox.getHiveKey
+        self.popHiveKey = self.prox.popHiveKey
+        self.getHiveKeys = self.prox.getHiveKeys
+        self.getUserDef = self.prox.getUserDef
+
+    async def _initLayr(self, layrinfo):
+        iden = layrinfo.get('iden')
+        ctorname = layrinfo.get('ctor')
+
+        ctor = s_dyndeps.tryDynLocal(ctorname)
+
+        layrinfo['readonly'] = True
+
+        layr = await self._ctorLayr(ctor, layrinfo)
+
+        self.onfini(layr)
+
+        self.layers[iden] = layr
+
+    async def _ctorLayr(self, ctor, layrinfo):
+        iden = layrinfo.get('iden')
+        layrdirn = s_common.genpath(self.dirn, 'layers', iden)
+        layr = await ctor.anit(layrinfo, layrdirn)
+        return layr
+
+    async def dyncall(self, iden, todo, gatekeys=()):
+        return await self.prox.dyncall(iden, todo, gatekeys=gatekeys)
+
+    async def dyniter(self, iden, todo, gatekeys=()):
+        async for item in self.prox.dyniter(iden, todo, gatekeys=gatekeys):
+            yield item
 
     def _logStormQuery(self, text, user):
         '''
@@ -400,17 +435,97 @@ class SpawnCore(s_base.Base):
             lvl = self.conf.get('storm:log:level')
             logger.log(lvl, 'Executing spawn storm query {%s} as [%s] from [%s]', text, user.name, self.pid)
 
-    def getStormCmd(self, name):
-        return self.stormcmds.get(name)
+    async def addStormPkg(self, pkgdef):
+        '''
+        Do it for the proxy, then myself
+        '''
+        todo = s_common.todo('addStormPkg', pkgdef)
+        await self.dyncall('cortex', todo)
 
-    async def getStormMods(self):
-        return self.stormmods
+        await self.loadStormPkg(pkgdef)
 
-    def getStormLib(self, path):
-        root = self.libroot
-        for name in path:
-            step = root[1].get(name)
-            if step is None:
-                return None
-            root = step
-        return root
+    async def delStormPkg(self, name):
+        '''
+        Do it for the proxy, then myself
+        '''
+        todo = s_common.todo('delStormPkg', name)
+        await self.dyncall('cortex', todo)
+
+        pkgdef = self.stormpkgs.get(name)
+        if pkgdef is None:
+            return
+
+        await self._dropStormPkg(pkgdef)
+
+    async def bumpSpawnPool(self):
+        pass
+
+    async def getStormPkgs(self):
+        return list(self.stormpkgs.values())
+
+    async def _addStormSvc(self, sdef):
+
+        iden = sdef.get('iden')
+        ssvc = self.svcsbyiden.get(iden)
+        if ssvc is not None:
+            return ssvc.sdef
+
+        ssvc = await self._setStormSvc(sdef)
+
+        return ssvc.sdef
+
+    async def _delStormSvcPkgs(self, iden):
+        '''
+        For now don't actually run this in the spawn case. This only needs to be
+        done in the master Cortex, not in spawns. Deleting a storm service package
+        from a spawn should not be making persistent changes.
+        '''
+        pass
+
+    async def _hndladdStormPkg(self, pdef):
+        '''
+        For now don't actually run this in the spawn case. This only needs to be
+        done in the master Cortex, not in spawns. Adding a storm service package
+        from a spawn should not be making persistent changes.
+        '''
+        # Note - this represents the bottom half of addStormPkg which is made
+        # via the @s_nexus.Pusher.onPushAuto('pkg:add') decorator.
+        pass
+
+    async def setStormSvcEvents(self, iden, edef):
+        svc = self.svcsbyiden.get(iden)
+        if svc is None:
+            mesg = f'No storm service with iden: {iden}'
+            raise s_exc.NoSuchStormSvc(mesg=mesg)
+
+        sdef = svc.sdef
+
+        sdef['evts'] = edef
+        return sdef
+
+    # A little selective inheritance
+    # TODO:  restructure cortex to avoid this hackery
+    _setStormSvc = s_cortex.Cortex._setStormSvc
+    _confirmStormPkg = s_cortex.Cortex._confirmStormPkg
+    _dropStormPkg = s_cortex.Cortex._dropStormPkg
+    _reqStormCmd = s_cortex.Cortex._reqStormCmd
+    _setStormCmd = s_cortex.Cortex._setStormCmd
+    _tryLoadStormPkg = s_cortex.Cortex._tryLoadStormPkg
+    _trySetStormCmd = s_cortex.Cortex._trySetStormCmd
+    addStormCmd = s_cortex.Cortex.addStormCmd
+    getDataModel = s_cortex.Cortex.getDataModel
+    getStormCmd = s_cortex.Cortex.getStormCmd
+    getStormCmds = s_cortex.Cortex.getStormCmds
+    getStormLib = s_cortex.Cortex.getStormLib
+    getStormMods = s_cortex.Cortex.getStormMods
+    getStormPkg = s_cortex.Cortex.getStormPkg
+    getStormQuery = s_cortex.Cortex.getStormQuery
+    getStormSvc = s_cortex.Cortex.getStormSvc
+    loadStormPkg = s_cortex.Cortex.loadStormPkg
+
+    _initStormCmds = s_cortex.Cortex._initStormCmds
+    _initStormOpts = s_cortex.Cortex._initStormOpts
+
+    _viewFromOpts = s_cortex.Cortex._viewFromOpts
+    _userFromOpts = s_cortex.Cortex._userFromOpts
+    getView = s_cortex.Cortex.getView
