@@ -12,7 +12,7 @@ def parse(text, base=None, chop=False):
     '''
     Parse a time string into an epoch millis value.
     '''
-    #TODO: use base to facilitate relative time offsets
+    # TODO: use base to facilitate relative time offsets
     text = text.strip().lower()
     text = (''.join([c for c in text if c.isdigit()]))
 
@@ -40,7 +40,7 @@ def parse(text, base=None, chop=False):
     elif tlen == 14:
         dt = datetime.datetime.strptime(text, '%Y%m%d%H%M%S')
 
-    elif tlen in (15, 16, 17):
+    elif 15 <= tlen <= 20:
         dt = datetime.datetime.strptime(text, '%Y%m%d%H%M%S%f')
 
     else:
