@@ -7,19 +7,12 @@ import synapse.lib.types as s_types
 import synapse.lib.module as s_module
 import synapse.lookup.pe as s_l_pe
 
-class FileBase(s_types.StrBase):
+class FileBase(s_types.Str):
 
     def postTypeInit(self):
-        s_types.StrBase.postTypeInit(self)
+        s_types.Str.postTypeInit(self)
         self.setNormFunc(str, self._normPyStr)
 
-    def indxByPref(self, valu):
-        valu = valu.strip().lower().replace('\\', '/')
-        indx = valu.encode('utf8', 'surrogatepass')[:248]
-        return (
-            ('pref', indx),
-        )
-
     def _normPyStr(self, valu):
 
         norm = valu.strip().lower().replace('\\', '/')
@@ -33,19 +26,12 @@ class FileBase(s_types.StrBase):
 
         return norm, {'subs': subs}
 
-class FilePath(s_types.StrBase):
+class FilePath(s_types.Str):
 
     def postTypeInit(self):
-        s_types.StrBase.postTypeInit(self)
+        s_types.Str.postTypeInit(self)
         self.setNormFunc(str, self._normPyStr)
 
-    def indxByPref(self, valu):
-        valu = valu.strip().lower().replace('\\', '/')
-        indx = valu.encode('utf8', 'surrogatepass')[:248]
-        return (
-            ('pref', indx),
-        )
-
     def _normPyStr(self, valu):
 
         if len(valu) == 0:
@@ -88,18 +74,15 @@ class FilePath(s_types.StrBase):
 
         return fullpath, {'subs': subs}
 
-class FileBytes(s_types.Type):
+class FileBytes(s_types.Str):
 
     def postTypeInit(self):
+        s_types.Str.postTypeInit(self)
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(list, self._normPyList)
         self.setNormFunc(tuple, self._normPyList)
         self.setNormFunc(bytes, self._normPyBytes)
 
-    def indx(self, norm):
-        # impossible for the normed value to be too long for indx
-        return norm.encode('utf8')
-
     def _normPyList(self, valu):
         guid, info = self.modl.type('guid').norm(valu)
         norm = f'guid:{guid}'
@@ -287,7 +270,6 @@ class FileModule(s_module.CoreModule):
                         'doc': 'The best known base name for the file.'}),
 
                     ('mime', ('file:mime', {}), {
-                        'defval': '??',
                         'doc': 'The "best" mime type name for the file.'}),
 
                     ('mime:x509:cn', ('str', {}), {
