@@ -1,6 +1,7 @@
 import synapse.exc as s_exc
 
 import synapse.lib.gis as s_gis
+import synapse.lib.layer as s_layer
 import synapse.lib.types as s_types
 import synapse.lib.module as s_module
 import synapse.lib.grammar as s_grammar
@@ -21,9 +22,150 @@ distrepr = (
     (10.0, 'cm'),
 )
 
-class Dist(s_types.IntBase):
+geojsonschema = {
+
+    'definitions': {
+
+        'BoundingBox': {'type': 'array', 'minItems': 4, 'items': {'type': 'number'}},
+        'PointCoordinates': {'type': 'array', 'minItems': 2, 'items': {'type': 'number'}},
+        'LineStringCoordinates': {'type': 'array', 'minItems': 2, 'items': {'$ref': '#/definitions/PointCoordinates'}},
+        'LinearRingCoordinates': {'type': 'array', 'minItems': 4, 'items': {'$ref': '#/definitions/PointCoordinates'}},
+        'PolygonCoordinates': {'type': 'array', 'items': {'$ref': '#/definitions/LinearRingCoordinates'}},
+
+        'Point': {
+            'title': 'GeoJSON Point',
+            'type': 'object',
+            'required': ['type', 'coordinates'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['Point']},
+                'coordinates': {'$ref': '#/definitions/PointCoordinates'},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+         },
+
+        'LineString': {
+            'title': 'GeoJSON LineString',
+            'type': 'object',
+            'required': ['type', 'coordinates'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['LineString']},
+                'coordinates': {'$ref': '#/definitions/LineStringCoordinates'},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+         },
+
+        'Polygon': {
+            'title': 'GeoJSON Polygon',
+            'type': 'object',
+            'required': ['type', 'coordinates'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['Polygon']},
+                'coordinates': {'$ref': '#/definitions/PolygonCoordinates'},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+        },
+
+        'MultiPoint': {
+            'title': 'GeoJSON MultiPoint',
+            'type': 'object',
+            'required': ['type', 'coordinates'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['MultiPoint']},
+                'coordinates': {'type': 'array', 'items': {'$ref': '#/definitions/PointCoordinates'}},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+        },
+
+        'MultiLineString': {
+            'title': 'GeoJSON MultiLineString',
+            'type': 'object',
+            'required': ['type', 'coordinates'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['MultiLineString']},
+                'coordinates': {'type': 'array', 'items': {'$ref': '#/definitions/LineStringCoordinates'}},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+         },
+
+        'MultiPolygon': {
+            'title': 'GeoJSON MultiPolygon',
+            'type': 'object',
+            'required': ['type', 'coordinates'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['MultiPolygon']},
+                'coordinates': {'type': 'array', 'items': {'$ref': '#/definitions/PolygonCoordinates'}},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+        },
+
+        'GeometryCollection': {
+            'title': 'GeoJSON GeometryCollection',
+            'type': 'object',
+            'required': ['type', 'geometries'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['GeometryCollection']},
+                'geometries': {'type': 'array', 'items': {'oneOf': [
+                    {'$ref': '#/definitions/Point'},
+                    {'$ref': '#/definitions/LineString'},
+                    {'$ref': '#/definitions/Polygon'},
+                    {'$ref': '#/definitions/MultiPoint'},
+                    {'$ref': '#/definitions/MultiLineString'},
+                    {'$ref': '#/definitions/MultiPolygon'},
+                ]}},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+        },
+
+        'Feature': {
+            'title': 'GeoJSON Feature',
+            'type': 'object',
+            'required': ['type', 'properties', 'geometry'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['Feature']},
+                'geometry': {'oneOf': [
+                    {'type': 'null'},
+                    {'$ref': '#/definitions/Point'},
+                    {'$ref': '#/definitions/LineString'},
+                    {'$ref': '#/definitions/Polygon'},
+                    {'$ref': '#/definitions/MultiPoint'},
+                    {'$ref': '#/definitions/MultiLineString'},
+                    {'$ref': '#/definitions/MultiPolygon'},
+                    {'$ref': '#/definitions/GeometryCollection'},
+                ]},
+                'properties': {'oneOf': [{'type': 'null'}, {'type': 'object'}]},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+        },
+
+        'FeatureCollection': {
+            'title': 'GeoJSON FeatureCollection',
+            'type': 'object',
+            'required': ['type', 'features'],
+            'properties': {
+                'type': {'type': 'string', 'enum': ['FeatureCollection']},
+                'features': {'type': 'array', 'items': {'$ref': '#/definitions/Feature'}},
+                'bbox': {'$ref': '#/definitions/BoundingBox'},
+            },
+        },
+    },
+
+    'oneOf': [
+        {'$ref': '#/definitions/Point'},
+        {'$ref': '#/definitions/LineString'},
+        {'$ref': '#/definitions/Polygon'},
+        {'$ref': '#/definitions/MultiPoint'},
+        {'$ref': '#/definitions/MultiLineString'},
+        {'$ref': '#/definitions/MultiPolygon'},
+        {'$ref': '#/definitions/GeometryCollection'},
+        {'$ref': '#/definitions/Feature'},
+        {'$ref': '#/definitions/FeatureCollection'},
+    ],
+}
+
+class Dist(s_types.Int):
 
     def postTypeInit(self):
+        s_types.Int.postTypeInit(self)
         self.setNormFunc(int, self._normPyInt)
         self.setNormFunc(str, self._normPyStr)
 
@@ -47,9 +189,6 @@ class Dist(s_types.IntBase):
 
         return int(valu * mult), {}
 
-    def indx(self, norm):
-        return norm.to_bytes(8, 'big')
-
     def repr(self, norm):
 
         for base, unit in distrepr:
@@ -59,45 +198,19 @@ class Dist(s_types.IntBase):
 
         return '%d mm' % (norm,)
 
-class Latitude(s_types.Type):
-    SCALE = 10**8  # ~1mm resolution
-    SPACE = 90 * 10**8
-
-    def postTypeInit(self):
-        self.setNormFunc(int, self._normIntStr)
-        self.setNormFunc(str, self._normIntStr)
-        self.setNormFunc(float, self._normFloat)
-
-    def _normFloat(self, valu):
-        if valu > 90.0 or valu < -90.0:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg='Latitude may only be -90.0 to 90.0')
-
-        valu = round(valu, 8)
-
-        return valu, {}
-
-    def _normIntStr(self, valu):
-        try:
-            valu = float(valu)
-        except Exception:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg='Invalid float format')
-        return self._normFloat(valu)
-
-    def indx(self, norm):
-        return int(norm * Latitude.SCALE + Latitude.SPACE).to_bytes(5, 'big')
-
 class LatLong(s_types.Type):
 
+    stortype = s_layer.STOR_TYPE_LATLONG
+
     def postTypeInit(self):
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(list, self._normPyTuple)
         self.setNormFunc(tuple, self._normPyTuple)
 
         self.setCmprCtor('near=', self._cmprNear)
-        self.setLiftHintCmprCtor('near=', self._cmprNear)
-        self.indxcmpr['near='] = self._indxNear
+        self.storlifts.update({
+            'near=': self._storLiftNear,
+        })
 
     def _normCmprValu(self, valu):
         latlong, dist = valu
@@ -107,25 +220,17 @@ class LatLong(s_types.Type):
 
     def _cmprNear(self, valu):
         latlong, dist = self._normCmprValu(valu)
+
         def cmpr(valu):
             if s_gis.haversine(valu, latlong) <= dist:
                 return True
             return False
         return cmpr
 
-    def _indxNear(self, valu):
-        (lat, long), dist = self._normCmprValu(valu)
-        latmin, latmax, longmin, longmax = s_gis.bbox(lat, long, dist)
-
-        latmin, longmin = self.norm((latmin, longmin))[0]
-        latmax, longmax = self.norm((latmax, longmax))[0]
-
-        minv = self.indx((latmin, longmin))
-        maxv = self.indx((latmax, longmax))
-
-        return (
-            ('range', (minv, maxv)),
-        )
+    def _storLiftNear(self, cmpr, valu):
+        latlong = self.norm(valu[0])[0]
+        dist = self.modl.type('geo:dist').norm(valu[1])[0]
+        return ((cmpr, (latlong, dist), self.stortype),)
 
     def _normPyStr(self, valu):
         valu = tuple(valu.strip().split(','))
@@ -145,42 +250,9 @@ class LatLong(s_types.Type):
 
         return (latv, lonv), {'subs': {'lat': latv, 'lon': lonv}}
 
-    def indx(self, valu):
-        return self.modl.type('geo:latitude').indx(valu[0]) + self.modl.type('geo:longitude').indx(valu[1])
-
     def repr(self, norm):
         return f'{norm[0]},{norm[1]}'
 
-class Longitude(s_types.Type):
-    SCALE = 10**8  # ~1mm resolution
-    SPACE = 180 * 10**8
-
-    def postTypeInit(self):
-        self.setNormFunc(int, self._normIntStr)
-        self.setNormFunc(str, self._normIntStr)
-        self.setNormFunc(float, self._normFloat)
-
-    def _normIntStr(self, valu):
-        try:
-            valu = float(valu)
-        except Exception:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg='Invalid float format')
-        return self._normFloat(valu)
-
-    def _normFloat(self, valu):
-
-        if valu > 180.0 or valu < -180.0:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg='Longitude may only be -180.0 to 180.0')
-
-        valu = round(valu, 8)
-
-        return valu, {}
-
-    def indx(self, norm):
-        return int(norm * Longitude.SCALE + Longitude.SPACE).to_bytes(5, 'big')
-
 class GeoModule(s_module.CoreModule):
 
     def getModelDefs(self):
@@ -191,8 +263,6 @@ class GeoModule(s_module.CoreModule):
                     ('geo:dist', 'synapse.models.geospace.Dist', {}, {
                         'doc': 'A geographic distance (base unit is mm)', 'ex': '10 km'
                     }),
-                    ('geo:latitude', 'synapse.models.geospace.Latitude', {}, {}),
-                    ('geo:longitude', 'synapse.models.geospace.Longitude', {}, {}),
                     ('geo:latlong', 'synapse.models.geospace.LatLong', {}, {
                         'doc': 'A Lat/Long string specifying a point on Earth',
                         'ex': '-12.45,56.78'
@@ -205,12 +275,20 @@ class GeoModule(s_module.CoreModule):
                         'doc': 'Records a node latitude/longitude in space-time.'
                     }),
 
+                    ('geo:json', ('data', {'schema': geojsonschema}), {
+                        'doc': 'GeoJSON structured JSON data.',
+                    }),
+
                     ('geo:place', ('guid', {}), {
                         'doc': 'A GUID for a geographic place.'}),
 
                     ('geo:address', ('str', {'lower': 1, 'onespace': 1, 'strip': True}), {
                         'doc': 'A street/mailing address string.',
                     }),
+                    ('geo:longitude', ('float', {'min': -180.0, 'max': 180.0,
+                                       'minisvalid': False, 'maxisvalid': True}), {}),
+                    ('geo:latitude', ('float', {'min': -90.0, 'max': 90.0,
+                                      'minisvalid': True, 'maxisvalid': True}), {}),
 
                     ('geo:bbox', ('comp', {'sepr': ',', 'fields': (
                                                 ('xmin', 'geo:longitude'),
@@ -262,6 +340,9 @@ class GeoModule(s_module.CoreModule):
                         ('address', ('geo:address', {}), {
                             'doc': 'The street/mailing address for the place.'}),
 
+                        ('geojson', ('geo:json', {}), {
+                            'doc': 'A GeoJSON representation of the place.'}),
+
                         ('latlong', ('geo:latlong', {}), {
                             'doc': 'The lat/long position for the place.'}),
 
