@@ -3,7 +3,6 @@ An RMI framework for synapse.
 '''
 
 import os
-import ssl
 import asyncio
 import logging
 import collections
@@ -202,6 +201,7 @@ class GenrIter:
 
         async for item in genr:
             yield item
+            await asyncio.sleep(0)
 
     def __iter__(self):
         genr = s_glob.sync(self.proxy.task(self.todo, name=self.share))
@@ -419,10 +419,9 @@ class Proxy(s_base.Base):
     async def taskv2(self, todo, name=None):
 
         mesg = ('t2:init', {
-                    'todo': todo,
-                    'name': name,
-                    'sess': self.sess,
-        })
+                'todo': todo,
+                'name': name,
+                'sess': self.sess})
 
         link = await self.getPoolLink()
 
@@ -430,7 +429,7 @@ class Proxy(s_base.Base):
 
         mesg = await link.rx()
         if mesg is None:
-            return
+            raise s_exc.LinkShutDown(mesg='Remote peer disconnected')
 
         if mesg[0] == 't2:fini':
             await self._putPoolLink(link)
@@ -485,10 +484,9 @@ class Proxy(s_base.Base):
         task = Task()
 
         mesg = ('task:init', {
-                    'task': task.iden,
-                    'todo': todo,
-                    'name': name,
-        })
+                'task': task.iden,
+                'todo': todo,
+                'name': name, })
 
         self.tasks[task.iden] = task
 
@@ -597,11 +595,16 @@ class Client(s_base.Base):
     '''
     A Telepath client object which reconnects and allows waiting for link up.
 
-    conf = {
-        'timeout': 10,
-        'retrysleep': 0.2,
-        'link_poolsize': 4,
-    }
+    Notes:
+
+        The conf data allows changing parameters such as timeouts, retry period, and link pool size. The default
+        conf data can be seen below::
+
+            conf = {
+                'timeout': 10,
+                'retrysleep': 0.2,
+                'link_poolsize': 4,
+            }
 
     '''
     async def __anit__(self, url, opts=None, conf=None, onlink=None):
@@ -620,8 +623,6 @@ class Client(s_base.Base):
         self._t_opts = opts
         self._t_conf = conf
 
-        #TODO chop user/passwd out of url and set in opts
-
         self._t_proxy = None
         self._t_ready = asyncio.Event()
         self._t_onlink = onlink
@@ -683,6 +684,7 @@ class Client(s_base.Base):
             await self._t_proxy.fini()
 
         self._t_proxy = await openurl(url, **self._t_opts)
+
         async def fini():
             await self._fireLinkLoop()
 
