@@ -1,7 +1,10 @@
 import json
+
 import synapse.exc as s_exc
+import synapse.common as s_common
 
 import synapse.lib.ast as s_ast
+
 import synapse.tests.utils as s_test
 
 foo_stormpkg = {
@@ -41,8 +44,8 @@ foo_stormpkg = {
         {
             'name': 'importnest',
             'storm': '''
-            $counter = 0
-            $foobar = 0
+            $foobar = $(0)
+            $counter = $(0)
 
             function inner(arg2, add) {
                 $foobar = $( $foobar + $add )
@@ -128,6 +131,61 @@ class AstTest(s_test.SynTest):
             self.len(1, nodes)
             self.eq(nodes[0].get('tick'), 1546300800000)
 
+    async def test_ast_autoadd(self):
+
+        async with self.getTestCore() as core:
+            opts = {'mode': 'autoadd'}
+            nodes = await core.nodes('1.2.3.4 woot.com visi@vertex.link', opts=opts)
+            self.len(3, nodes)
+            self.eq(nodes[0].ndef, ('inet:ipv4', 0x01020304))
+            self.eq(nodes[1].ndef, ('inet:fqdn', 'woot.com'))
+            self.eq(nodes[2].ndef, ('inet:email', 'visi@vertex.link'))
+
+    async def test_ast_lookup(self):
+
+        async with self.getTestCore() as core:
+            nodes = await core.nodes('''[
+                inet:ipv4=1.2.3.4
+                inet:fqdn=foo.bar.com
+                inet:email=visi@vertex.link
+                inet:url="https://[ff::00]:4443/hehe?foo=bar&baz=faz"
+            ]''')
+            ndefs = [n.ndef for n in nodes]
+            self.len(4, ndefs)
+
+            opts = {'mode': 'lookup'}
+            nodes = await core.nodes('1.2.3.4 foo.bar.com visi@vertex.link https://[ff::00]:4443/hehe?foo=bar&baz=faz', opts=opts)
+            self.eq(ndefs, [n.ndef for n in nodes])
+
+            nodes = await core.nodes('1.2.3.4 foo.bar.com visi@vertex.link https://[ff::00]:4443/hehe?foo=bar&baz=faz | [ +#hehe ]', opts=opts)
+            self.len(4, nodes)
+            self.eq(ndefs, [n.ndef for n in nodes])
+            self.true(all(n.tags.get('hehe') is not None for n in nodes))
+
+            # AST object passes through inbound genrs
+            await core.nodes('[test:str=beep]')
+            beep_opts = {'ndefs': [('test:str', 'beep')], 'mode': 'lookup'}
+            nodes = await core.nodes('foo.bar.com | [+#beep]', beep_opts)
+            self.len(2, nodes)
+            self.eq({('test:str', 'beep'), ('inet:fqdn', 'foo.bar.com')},
+                    {n.ndef for n in nodes})
+            self.true(all([n.tags.get('beep') for n in nodes]))
+
+            # The lookup mode must get *something* to parse.
+            self.len(0, await core.nodes('', opts))
+
+            # The lookup must be *before* anything else, otherwise we
+            # parse it as a cmd name.
+            with self.raises(s_exc.NoSuchName):
+                await core.nodes('[+#thebeforetimes] | foo.bar.com', opts)
+
+            # And it works remotely
+            async with core.getLocalProxy() as prox:
+                msgs = await s_test.alist(prox.storm('1.2.3.4', opts))
+                nodes = [m[1] for m in msgs if m[0] == 'node']
+                self.len(1, nodes)
+                self.eq(nodes[0][0], ('inet:ipv4', 0x01020304))
+
     async def test_ast_subq_vars(self):
 
         async with self.getTestCore() as core:
@@ -140,7 +198,7 @@ class AstTest(s_test.SynTest):
                 { -> test:int [ :loc=haha ] $loc=:loc }
                 $lib.print($loc)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('haha', msgs)
 
             # Show that a computed variable being smashed by a
@@ -158,7 +216,7 @@ class AstTest(s_test.SynTest):
                 $lib.print($loc)
                 -test:comp
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('sol', msgs)
             self.stormIsInPrint('mars', msgs)
 
@@ -168,12 +226,14 @@ class AstTest(s_test.SynTest):
         '''
         async with self.getTestCore() as core:
             q = '''
-            [test:str=another]
-            $s = $lib.text("Foo")
-            $newvar=:hehe -.created
-            $s.add("yar {x}", x=$newvar)
-            $lib.print($s.str())'''
-            mesgs = await core.streamstorm(q).list()
+                [test:str=another :hehe=asdf]
+                $s = $lib.text("Foo")
+                $newvar=:hehe
+                -.created
+                $s.add("yar {x}", x=$newvar)
+                $lib.print($s.str())
+            '''
+            mesgs = await core.stormlist(q)
             prints = [m[1]['mesg'] for m in mesgs if m[0] == 'print']
             self.eq(['Foo'], prints)
 
@@ -286,7 +346,7 @@ class AstTest(s_test.SynTest):
 
             user = await core.auth.addUser('newb')
             with self.raises(s_exc.AuthDeny):
-                await core.nodes('[ (inet:ipv4=1.2.3.4 :asn=20) ]', user=user)
+                await core.nodes('[ (inet:ipv4=1.2.3.4 :asn=20) ]', opts={'user': user.iden})
 
     async def test_subquery_yield(self):
 
@@ -416,15 +476,25 @@ class AstTest(s_test.SynTest):
             $key = nope
             [ test:str=$data.$key ]
             '''
-            mesgs = await s_test.alist(core.streamstorm(q))
+            mesgs = await core.stormlist(q)
             errs = [m[1] for m in mesgs if m[0] == 'err']
-            self.eq(errs[0][0], 'BadPropValu')
+            self.eq(errs[0][0], 'BadTypeValu')
 
     async def test_ast_array_pivot(self):
 
         async with self.getTestCore() as core:
 
             nodes = await core.nodes('[ test:arrayprop="*" :ints=(1, 2, 3) ]')
+            self.len(1, nodes)
+
+            # Check that subs were added
+            nodes = await core.nodes('test:int=1')
+            self.len(1, nodes)
+            nodes = await core.nodes('test:int=2')
+            self.len(1, nodes)
+            nodes = await core.nodes('test:int=3')
+            self.len(1, nodes)
+
             nodes = await core.nodes('test:arrayprop -> *')
             self.len(3, nodes)
 
@@ -434,6 +504,12 @@ class AstTest(s_test.SynTest):
             nodes = await core.nodes('test:arrayprop:ints -> test:int')
             self.len(3, nodes)
 
+            nodes = await core.nodes('test:arrayprop:ints :ints -> test:int')
+            self.len(3, nodes)
+
+            nodes = await core.nodes('test:arrayprop:ints :ints -> *')
+            self.len(3, nodes)
+
             nodes = await core.nodes('test:arrayprop:ints -> *')
             self.len(3, nodes)
 
@@ -520,6 +596,56 @@ class AstTest(s_test.SynTest):
             self.len(1, nodes)
             self.eq(nodes[0].get('ints'), (100, 101, 102))
 
+    async def test_ast_array_addsub(self):
+
+        async with self.getTestCore() as core:
+
+            guid = s_common.guid()
+            nodes = await core.nodes(f'[ test:arrayprop={guid} ]')
+
+            # test starting with the property unset
+            nodes = await core.nodes(f'test:arrayprop={guid} [ :ints+=99 ]')
+            self.eq((99,), nodes[0].get('ints'))
+
+            # test that removing a non-existant item is ok...
+            nodes = await core.nodes(f'test:arrayprop={guid} [ :ints-=22 ]')
+
+            nodes = await core.nodes(f'test:arrayprop={guid} [ :ints-=99 ]')
+            self.eq((), nodes[0].get('ints'))
+
+            nodes = await core.nodes(f'test:arrayprop={guid} [ :ints=(1, 2, 3) ]')
+
+            nodes = await core.nodes(f'test:arrayprop={guid} [ :ints+=4 ]')
+            self.eq((1, 2, 3, 4), nodes[0].get('ints'))
+
+            nodes = await core.nodes(f'test:arrayprop={guid} [ :ints-=3 ]')
+            self.eq((1, 2, 4), nodes[0].get('ints'))
+
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes(f'test:arrayprop={guid} [ :ints+=asdf ]')
+
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes(f'test:arrayprop={guid} [ :ints-=asdf ]')
+
+            await core.nodes(f'test:arrayprop={guid} [ :ints?-=asdf ]')
+            self.eq((1, 2, 4), nodes[0].get('ints'))
+
+            await core.nodes(f'test:arrayprop={guid} [ :ints?+=asdf ]')
+            self.eq((1, 2, 4), nodes[0].get('ints'))
+
+            # ensure that we get a proper exception when using += (et al) on non-array props
+            with self.raises(s_exc.StormRuntimeError):
+                nodes = await core.nodes(f'[ inet:ipv4=1.2.3.4 :asn+=10 ]')
+
+            with self.raises(s_exc.StormRuntimeError):
+                nodes = await core.nodes(f'[ inet:ipv4=1.2.3.4 :asn?+=10 ]')
+
+            with self.raises(s_exc.StormRuntimeError):
+                nodes = await core.nodes(f'[ inet:ipv4=1.2.3.4 :asn-=10 ]')
+
+            with self.raises(s_exc.StormRuntimeError):
+                nodes = await core.nodes(f'[ inet:ipv4=1.2.3.4 :asn?-=10 ]')
+
     async def test_ast_del_array(self):
 
         async with self.getTestCore() as core:
@@ -589,29 +715,29 @@ class AstTest(s_test.SynTest):
             self.eq({('test:str', 'asdf'), ('test:str', 'qwer')},
                     {n.ndef for n in nodes})
 
-            msgs = await core.streamstorm('pkg.list').list()
+            msgs = await core.stormlist('pkg.list')
             self.stormIsInPrint('foo                             : (0, 0, 1)', msgs)
 
-            msgs = await core.streamstorm('pkg.del asdf').list()
+            msgs = await core.stormlist('pkg.del asdf')
             self.stormIsInPrint('No package names match "asdf". Aborting.', msgs)
 
             await core.addStormPkg(otherpkg)
-            msgs = await core.streamstorm('pkg.list').list()
+            msgs = await core.stormlist('pkg.list')
             self.stormIsInPrint('foosball', msgs)
 
-            msgs = await core.streamstorm('pkg.del foo').list()
+            msgs = await core.stormlist('pkg.del foo')
             self.stormIsInPrint('Multiple package names match "foo". Aborting.', msgs)
 
-            msgs = await core.streamstorm(f'pkg.del foosball').list()
+            msgs = await core.stormlist(f'pkg.del foosball')
             self.stormIsInPrint('Removing package: foosball', msgs)
 
-            msgs = await core.streamstorm(f'pkg.del foo').list()
+            msgs = await core.stormlist(f'pkg.del foo')
             self.stormIsInPrint('Removing package: foo', msgs)
 
             # Direct add via stormtypes
-            await core.streamstorm('$lib.pkg.add($pkg)',
-                                   opts={'vars': {'pkg': stormpkg}}).list()
-            msgs = await core.streamstorm('pkg.list').list()
+            await core.stormlist('$lib.pkg.add($pkg)',
+                                   opts={'vars': {'pkg': stormpkg}})
+            msgs = await core.stormlist('pkg.list')
             self.stormIsInPrint('stormpkg', msgs)
 
             with self.raises(s_exc.NoSuchName):
@@ -629,7 +755,7 @@ class AstTest(s_test.SynTest):
             $retn=$hello()
             $lib.print('retn is: {retn}', retn=$retn)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('retn is: hello', msgs)
 
             # Simple echo function
@@ -641,7 +767,7 @@ class AstTest(s_test.SynTest):
             $retn=$echo($node.value())
             $lib.print('retn is: {retn}', retn=$retn)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('retn is: foo', msgs)
             self.stormIsInPrint('retn is: bar', msgs)
 
@@ -657,7 +783,7 @@ class AstTest(s_test.SynTest):
             $retn=$echo($node.value())
             $lib.print('retn is: {retn}', retn=$retn)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('arg is foo', msgs)
             self.stormIsInPrint('arg is bar', msgs)
             self.stormIsInPrint('retn is: 1234', msgs)
@@ -674,7 +800,7 @@ class AstTest(s_test.SynTest):
             $retn=$cond($node.value())
             $lib.print('retn is: {retn}', retn=$retn)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('retn is: None', msgs)
             self.stormIsInPrint('retn is: 1', msgs)
 
@@ -683,7 +809,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint('hello', 'world', arg3='goodbye')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('arg1: hello', msgs)
             self.stormIsInPrint('arg2: world', msgs)
             self.stormIsInPrint('arg3: goodbye', msgs)
@@ -692,7 +818,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint(arg3='goodbye', arg1='hello', arg2='world')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('arg1: hello', msgs)
             self.stormIsInPrint('arg2: world', msgs)
             self.stormIsInPrint('arg3: goodbye', msgs)
@@ -711,7 +837,7 @@ class AstTest(s_test.SynTest):
             $output = $outer()
             $lib.print($output)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('inner vertex', msgs)
             self.stormIsInPrint('foobarbazbiz', msgs)
 
@@ -728,7 +854,7 @@ class AstTest(s_test.SynTest):
             $lib.print('output is {a}', a=$output)
             '''
 
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('output is 18', msgs)
 
             # recursive functions
@@ -743,7 +869,7 @@ class AstTest(s_test.SynTest):
             $lib.print('final recursive output is {out}', out=$output)
             '''
 
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('final recursive output is 6', msgs)
 
             # return a function (not a value, but a ref to the function itself)
@@ -764,7 +890,7 @@ class AstTest(s_test.SynTest):
             $output = $func()
             $lib.print('[{now}, "got {out}"]', now=$($lib.time.now()), out=$output)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             prints = list(filter(lambda m: m[0] == 'print', msgs))
             self.eq(len(prints), 3)
 
@@ -801,7 +927,7 @@ class AstTest(s_test.SynTest):
             $lib.print($boop())
             $lib.print("biz is now {biz}", biz=$biz)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             prints = list(filter(lambda m: m[0] == 'print', msgs))
             self.len(6, prints)
             self.stormIsInPrint("neato burrito", msgs)
@@ -814,7 +940,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $lib.print($outer("1337"))
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             for msg in msgs:
                 self.ne('print', msg[0])
 
@@ -832,7 +958,7 @@ class AstTest(s_test.SynTest):
             $foo = $bar("hehe")
             $lib.print($foo)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('hehe', msgs)
             self.stormIsInPrint('arg1=hehe', msgs)
             self.stormIsInPrint('arg2=hehe', msgs)
@@ -840,15 +966,15 @@ class AstTest(s_test.SynTest):
             # call an import and have it's module local variables be mapped in to its own scope
             q = '''
             $test = $lib.import(importnest)
-            $haha = $test.outer(False, $(33))
+            $haha = $test.outer($lib.false, $(33))
             $lib.print($haha)
-            $hehe = $test.outer(True, $(17))
+            $hehe = $test.outer($lib.true, $(17))
             $lib.print($hehe)
-            $retn = $lib.import(importnest).outer(True, $(90))
+            $retn = $lib.import(importnest).outer($lib.true, $(90))
             $lib.print($retn)
-            $lib.print("counter is {c}", c$counter)
+            $lib.print("counter is {c}", c=$counter)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             prints = list(filter(lambda m: m[0] == 'print', msgs))
             self.len(9, prints)
             self.stormIsInPrint('counter is 0', msgs)
@@ -882,7 +1008,7 @@ class AstTest(s_test.SynTest):
             yield $foo("bloooop")
             $lib.print("nodes added: {c}", c=$count)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('nodes added: 1', msgs)
             self.stormIsInPrint('nodes added: 2', msgs)
 
@@ -902,7 +1028,7 @@ class AstTest(s_test.SynTest):
             $lib.print("retn is {ans}", ans=$( $foo($global)) )
             $lib.print("this should not print, but {wat}", wat=$wat)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             prints = list(filter(lambda m: m[0] == 'print', msgs))
             self.len(2, prints)
             self.stormIsInPrint('arg1 is 445', msgs)
@@ -924,7 +1050,7 @@ class AstTest(s_test.SynTest):
             $lib.print($override())
             $lib.print("NO OVERRIDES FOR YOU")
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('The new lib', msgs)
             self.stormIsInPrint('We should have inherited the one true lib', msgs)
             self.stormIsInPrint('Hi :)', msgs)
@@ -947,7 +1073,7 @@ class AstTest(s_test.SynTest):
             $lib.print($nooverride("recovered"))
             '''
 
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('neato', msgs)
             self.stormIsInPrint('foobar', msgs)
             self.stormIsInPrint('recovered', msgs)
@@ -959,7 +1085,7 @@ class AstTest(s_test.SynTest):
             $lib.print($node.value())
             $lib.print("splat shouldn't exist, but we got {s}", s=$splat)
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             self.stormIsInPrint('yieldsforimports', msgs)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'NoSuchVar')
@@ -970,7 +1096,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint('hello', 'world')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('name'), 'pprint')
@@ -982,7 +1108,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint(arg1='hello', arg2='world')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('name'), 'pprint')
@@ -994,7 +1120,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint('hello', 'world', arg3='world', arg4='newp')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('name'), 'pprint')
@@ -1005,7 +1131,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint('hello', 'world', arg1='hello')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('name'), 'pprint')
@@ -1016,7 +1142,7 @@ class AstTest(s_test.SynTest):
             $test=$lib.import(test)
             $haha=$test.pprint('hello', 'world', 'goodbye', 'newp')
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('name'), 'pprint')
@@ -1074,12 +1200,12 @@ class AstTest(s_test.SynTest):
                     '''
                     query = core.getStormQuery(q)
                     runt.loadRuntVars(query)
-                    async for item in query.run(runt, s_ast.agen()):
+                    async for item in query.run(runt, s_common.agen()):
                         pass
                     func = list(filter(lambda o: isinstance(o, s_ast.Function), query.kids))[0]
                     oldfunc = runt.vars['lolol']
                     funcrunt = await runt.getScopeRuntime(func.kids[2])
-                    async for item in func.run(funcrunt, s_ast.agen()):
+                    async for item in func.run(funcrunt, s_common.agen()):
                         pass
                     funcrunt.globals.add('nope')
                     funcrunt.globals.add('lolol')
@@ -1121,7 +1247,7 @@ class AstTest(s_test.SynTest):
                 $lib.fire(event, dict=$dict)
             }
             '''
-            mesgs = await core.streamstorm(q).list()
+            mesgs = await core.stormlist(q)
             stormfire = [m for m in mesgs if m[0] == 'storm:fire']
             self.len(1, stormfire)
             self.eq(stormfire[0][1].get('data').get('dict'),
@@ -1132,22 +1258,11 @@ class AstTest(s_test.SynTest):
             $set=$lib.set()
             $set.foo="bar"
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('mesg'), 'Set does not support assignment.')
 
-            # Some types we have in the runtime cannot be converted to StormTypes
-            q = '''
-            function f(a){ return() }
-            $f.newp="newp"
-            '''
-            msgs = await core.streamstorm(q).list()
-            erfo = [m for m in msgs if m[0] == 'err'][0]
-            self.eq(erfo[1][0], 'NoSuchType')
-            self.eq(erfo[1][1].get('mesg'),
-                    'Unable to convert python primitive to StormType.')
-
     async def test_ast_initfini(self):
 
         async with self.getTestCore() as core:
@@ -1166,7 +1281,7 @@ class AstTest(s_test.SynTest):
                 fini { $lib.print('xfini: {x}', x=$x) }
             '''
 
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
 
             types = [m[0] for m in msgs if m[0] in ('node', 'print', 'storm:fire')]
             self.eq(types, ('storm:fire', 'node', 'node', 'print'))
@@ -1201,7 +1316,7 @@ class AstTest(s_test.SynTest):
                 [test:str=$hehe]
             }
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('mesg'), 'Init block query must be runtsafe')
@@ -1241,7 +1356,7 @@ class AstTest(s_test.SynTest):
                 [(test:str=fini4 :hehe=$hehe)]
             }
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             erfo = [m for m in msgs if m[0] == 'err'][0]
             self.eq(erfo[1][0], 'StormRuntimeError')
             self.eq(erfo[1][1].get('mesg'), 'Fini block query must be runtsafe')
@@ -1258,8 +1373,103 @@ class AstTest(s_test.SynTest):
                 }
             }
             '''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             firs = [m for m in msgs if m[0] == 'storm:fire']
             self.len(1, firs)
             evnt = firs[0]
             self.eq(evnt[1].get('data'), {'total': 3})
+
+    async def test_ast_cmdargs(self):
+
+        async with self.getTestCore() as core:
+
+            scmd = {
+                'name': 'foo',
+                'cmdargs': (
+                    ('--bar', {}),
+                ),
+                'storm': '''
+                    $ival = $lib.cast(ival, $cmdopts.bar)
+                    [ test:str=1234 +#foo=$ival ]
+                ''',
+            }
+
+            await core.setStormCmd(scmd)
+
+            nodes = await core.nodes('foo --bar (2020,2021) | +#foo@=202002')
+            self.len(1, nodes)
+
+            scmd = {
+                'name': 'baz',
+                'cmdargs': (
+                    ('--faz', {}),
+                ),
+                'storm': '''
+                    // subquery forces per-node evaluation of even runt safe vars
+                    {
+                        $ival = $lib.cast(ival, $cmdopts.faz)
+                        [ test:str=beep +#foo=$ival ]
+                    }
+                ''',
+            }
+
+            await core.setStormCmd(scmd)
+
+            await core.nodes('[ test:int=5678 +#foo=(2018, 2021) ]')
+            await core.nodes('[ test:int=1111 +#foo=(1977, 2019) ]')
+
+            nodes = await core.nodes('test:int | baz --faz #foo')
+            self.len(2, nodes)
+
+            nodes = await core.nodes('test:str +#foo@=1984 +#foo@=202002')
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('test:str', 'beep'))
+
+    async def test_ast_pullone(self):
+
+        async def genr():
+            yield 1
+            yield 2
+            yield 3
+
+        vals = [x async for x in await s_ast.pullone(genr())]
+        self.eq((1, 2, 3), vals)
+
+        data = {}
+        async def empty():
+            data['executed'] = True
+            if 0: yield None
+
+        vals = [x async for x in await s_ast.pullone(empty())]
+        self.eq([], vals)
+        self.true(data.get('executed'))
+
+        async def hasone():
+            yield 1
+
+        vals = [x async for x in await s_ast.pullone(hasone())]
+        self.eq((1,), vals)
+
+    async def test_ast_expr(self):
+
+        async with self.getTestCore() as core:
+
+            nodes = await core.nodes('if (true) { [inet:ipv4=1.2.3.4] }')
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('inet:ipv4', 0x01020304))
+
+            nodes = await core.nodes('if (false) { [inet:ipv4=1.2.3.4] }')
+            self.len(0, nodes)
+
+            nodes = await core.nodes('[ test:int=(18 + 2) ]')
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('test:int', 20))
+
+    async def test_ast_subgraph_light_edges(self):
+        async with self.getTestCore() as core:
+            await core.nodes('[ test:int=20 <(refs)+ { [media:news=*] } ]')
+            msgs = await core.stormlist('media:news', opts={'graph': True})
+            nodes = [m[1] for m in msgs if m[0] == 'node']
+            self.len(1, nodes)
+            self.len(1, nodes[0][1]['path']['edges'])
+            self.eq('refs', nodes[0][1]['path']['edges'][0][1]['verb'])
