@@ -2,11 +2,13 @@ import os
 import sys
 import signal
 import asyncio
+import contextlib
 import multiprocessing
 
 import synapse.exc as s_exc
 
 import synapse.lib.base as s_base
+import synapse.lib.coro as s_coro
 
 import synapse.tests.utils as s_t_utils
 
@@ -35,6 +37,12 @@ class Hehe(s_base.Base):
         await s_base.Base.__anit__(self)
         self.foo = foo
         self.bar = self.foo + 10
+        self.posted = False
+
+    async def postAnit(self):
+        self.posted = True
+        if self.foo == -1:
+            raise s_exc.BadArg(mesg='boom')
 
 class BaseTest(s_t_utils.SynTest):
 
@@ -56,6 +64,11 @@ class BaseTest(s_t_utils.SynTest):
         afoo = await Hehe.anit(20)
         self.eq(afoo.foo, 20)
         self.eq(afoo.bar, 30)
+        self.true(afoo.posted)
+
+        with self.raises(s_exc.BadArg) as cm:
+            await Hehe.anit(-1)
+        self.eq(cm.exception.get('mesg'), 'boom')
 
     async def test_coro_fini(self):
 
@@ -72,6 +85,35 @@ class BaseTest(s_t_utils.SynTest):
         self.true(event.is_set())
         self.false(f._isExitExc())
 
+    async def test_enter_context(self):
+        state = None
+
+        @contextlib.contextmanager
+        def ctxtest():
+            nonlocal state
+            state = "before"
+            yield state
+            state = "after"
+
+        @contextlib.asynccontextmanager
+        async def actxtest():
+            nonlocal state
+            state = "before2"
+            yield state
+            state = "after2"
+
+        async with await s_base.Base.anit() as base:
+            await base.enter_context(ctxtest())
+            self.eq("before", state)
+
+        self.eq("after", state)
+
+        async with await s_base.Base.anit() as base:
+            await base.enter_context(actxtest())
+            self.eq("before2", state)
+
+        self.eq("after2", state)
+
     async def test_base_link(self):
 
         base1 = await s_base.Base.anit()
@@ -410,3 +452,39 @@ class BaseTest(s_t_utils.SynTest):
 
         self.true(mixed.m1fini.is_set())
         self.true(mixed.m2fini.is_set())
+
+    async def test_base_schedgenr(self):
+        async def goodgenr():
+            yield 'foo'
+            await asyncio.sleep(0)
+            yield 'bar'
+            await asyncio.sleep(0)
+
+        async def badgenr():
+            yield 'foo'
+            await asyncio.sleep(0)
+            raise s_exc.SynErr('rando')
+
+        async def slowgenr():
+            yield 'foo'
+            await asyncio.sleep(5)
+            yield 'bar'
+
+        event = asyncio.Event()
+
+        async def slowtask():
+            g = s_base.schedGenr(slowgenr())
+            item = await g.__anext__()
+            self.eq('foo', item)
+            event.set()
+            item = await g.__anext__()
+
+        items = [item async for item in s_base.schedGenr(goodgenr())]
+        self.eq(items, ['foo', 'bar'])
+        await self.agenraises(s_exc.SynErr, s_base.schedGenr(badgenr()))
+
+        async with await s_base.Base.anit() as base:
+            task = base.schedCoro(slowtask())
+            self.true(await s_coro.event_wait(event, 3.0))
+
+        await self.asyncraises(asyncio.CancelledError, task)
