@@ -8,19 +8,6 @@ import synapse.tests.utils as s_tests
 
 class HttpApiTest(s_tests.SynTest):
 
-    async def test_http_port(self):
-        async with self.getTestCore() as core:
-            # Run http instead of https for this test
-            host, port = await core.addHttpPort(0, host='127.0.0.1')
-
-            root = core.auth.getUserByName('root')
-            await root.setPasswd('secret')
-            async with aiohttp.ClientSession() as sess:
-                async with sess.get(f'http://root:secret@localhost:{port}/api/v1/auth/users') as resp:
-                    item = await resp.json()
-                    users = item.get('result')
-                    self.isin('root', [u.get('name') for u in users])
-
     async def test_reqauth(self):
 
         class ReqAuthHandler(s_httpapi.Handler):
@@ -34,7 +21,7 @@ class HttpApiTest(s_tests.SynTest):
 
             host, port = await core.addHttpsPort(0, host='127.0.0.1')
             url = f'https://localhost:{port}/api/tests/test_reqauth'
-            root = core.auth.getUserByName('root')
+            root = await core.auth.getUserByName('root')
             await root.setPasswd('secret')
 
             user = await core.auth.addUser('user')
@@ -76,7 +63,7 @@ class HttpApiTest(s_tests.SynTest):
 
             host, port = await core.addHttpsPort(0, host='127.0.0.1')
 
-            root = core.auth.getUserByName('root')
+            root = await core.auth.getUserByName('root')
             await root.setPasswd('secret')
 
             newb = await core.auth.addUser('newb')
@@ -93,7 +80,7 @@ class HttpApiTest(s_tests.SynTest):
                     retn = await resp.json()
                     self.eq('ok', retn.get('status'))
 
-                self.true(newb.locked)
+                self.true(newb.isLocked())
 
                 async with sess.get(f'https://localhost:{port}/api/v1/auth/users') as resp:
                     item = await resp.json()
@@ -136,7 +123,7 @@ class HttpApiTest(s_tests.SynTest):
 
             host, port = await core.addHttpsPort(0, host='127.0.0.1')
 
-            root = core.auth.getUserByName('root')
+            root = await core.auth.getUserByName('root')
             await root.setPasswd('secret')
 
             newb = await core.auth.addUser('bob')
@@ -144,7 +131,7 @@ class HttpApiTest(s_tests.SynTest):
 
             bobs = await core.auth.addRole('bobs')
 
-            await newb.grant('bobs')
+            await newb.grant(bobs.iden)
 
             async with self.getHttpSess() as sess:
 
@@ -173,7 +160,7 @@ class HttpApiTest(s_tests.SynTest):
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/delrole', data=b'asdf') as resp:
                     item = await resp.json()
                     self.eq('err', item.get('status'))
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 info = {'name': 'newp'}
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/delrole', json=info) as resp:
@@ -181,20 +168,21 @@ class HttpApiTest(s_tests.SynTest):
                     self.eq('err', item.get('status'))
                     self.eq('NoSuchRole', item.get('code'))
 
+                self.len(2, newb.getRoles())
                 info = {'name': 'bobs'}
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/delrole', json=info) as resp:
                     item = await resp.json()
                     self.eq('ok', item.get('status'))
 
-            self.len(0, newb.getRoles())
-            self.none(core.auth.getRoleByName('bobs'))
+            self.len(1, newb.getRoles())
+            self.none(await core.auth.getRoleByName('bobs'))
 
     async def test_http_passwd(self):
         async with self.getTestCore() as core:
 
             host, port = await core.addHttpsPort(0, host='127.0.0.1')
 
-            root = core.auth.getUserByName('root')
+            root = await core.auth.getUserByName('root')
             await root.setPasswd('secret')
 
             newb = await core.auth.addUser('newb')
@@ -245,97 +233,98 @@ class HttpApiTest(s_tests.SynTest):
         async with self.getTestCore() as core:
 
             host, port = await core.addHttpsPort(0, host='127.0.0.1')
-
-            core.insecure = True
+            root = await core.auth.getUserByName('root')
+            await root.setPasswd('root')
 
             async with self.getHttpSess() as sess:
 
                 info = {'name': 'visi', 'passwd': 'secret', 'admin': True}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/adduser', json=info) as resp:
+                # Make the first user as root
+                async with sess.post(f'https://root:root@localhost:{port}/api/v1/auth/adduser', json=info) as resp:
                     item = await resp.json()
                     self.nn(item.get('result').get('iden'))
                     visiiden = item['result']['iden']
 
                 info = {'name': 'noob', 'passwd': 'nooblet', 'email': 'nobody@nowhere.com'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/adduser', json=info) as resp:
+                # The visi user is an admin, so reuse it
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/adduser', json=info) as resp:
                     item = await resp.json()
                     self.nn(item.get('result').get('iden'))
                     self.eq('nobody@nowhere.com', item['result']['email'])
                     noobiden = item['result']['iden']
 
                 info = {'name': 'visi', 'passwd': 'secret', 'admin': True}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/adduser', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/adduser', json=info) as resp:
                     item = await resp.json()
                     self.eq('err', item.get('status'))
                     self.eq('DupUser', item.get('code'))
 
                 info = {'name': 'analysts'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/addrole', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/addrole', json=info) as resp:
                     item = await resp.json()
                     self.nn(item.get('result').get('iden'))
                     analystiden = item['result']['iden']
 
                 info = {'name': 'analysts'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/addrole', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/addrole', json=info) as resp:
                     item = await resp.json()
                     self.eq('err', item.get('status'))
                     self.eq('DupRole', item.get('code'))
 
-                async with sess.get(f'https://localhost:{port}/api/v1/auth/user/newp') as resp:
+                async with sess.get(f'https://visi:secret@localhost:{port}/api/v1/auth/user/newp') as resp:
                     item = await resp.json()
                     self.eq('NoSuchUser', item.get('code'))
 
-                async with sess.get(f'https://localhost:{port}/api/v1/auth/role/newp') as resp:
+                async with sess.get(f'https://visi:secret@localhost:{port}/api/v1/auth/role/newp') as resp:
                     item = await resp.json()
                     self.eq('NoSuchRole', item.get('code'))
 
-                async with sess.get(f'https://localhost:{port}/api/v1/auth/users') as resp:
+                async with sess.get(f'https://visi:secret@localhost:{port}/api/v1/auth/users') as resp:
                     item = await resp.json()
                     users = item.get('result')
                     self.isin('visi', [u.get('name') for u in users])
 
-                async with sess.get(f'https://localhost:{port}/api/v1/auth/roles') as resp:
+                async with sess.get(f'https://visi:secret@localhost:{port}/api/v1/auth/roles') as resp:
                     item = await resp.json()
                     roles = item.get('result')
                     self.isin('analysts', [r.get('name') for r in roles])
 
                 info = {'user': 'blah', 'role': 'blah'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/grant', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/grant', json=info) as resp:
                     item = await resp.json()
                     self.eq('NoSuchUser', item.get('code'))
 
                 info = {'user': visiiden, 'role': 'blah'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/grant', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/grant', json=info) as resp:
                     item = await resp.json()
                     self.eq('NoSuchRole', item.get('code'))
 
                 info = {'user': 'blah', 'role': 'blah'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/revoke', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/revoke', json=info) as resp:
                     item = await resp.json()
                     self.eq('NoSuchUser', item.get('code'))
 
                 info = {'user': visiiden, 'role': 'blah'}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/revoke', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/revoke', json=info) as resp:
                     item = await resp.json()
                     self.eq('NoSuchRole', item.get('code'))
 
                 info = {'user': visiiden, 'role': analystiden}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/grant', json=info) as resp:
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/grant', json=info) as resp:
+                    item = await resp.json()
+                    self.eq('ok', item.get('status'))
+                    roles = item['result']['roles']
+                    self.len(2, roles)
+                    self.isin(analystiden, roles)
+
+                info = {'user': visiiden, 'role': analystiden}
+                async with sess.post(f'https://visi:secret@localhost:{port}/api/v1/auth/revoke', json=info) as resp:
                     item = await resp.json()
                     self.eq('ok', item.get('status'))
                     roles = item['result']['roles']
                     self.len(1, roles)
-                    self.eq(analystiden, roles[0])
-
-                info = {'user': visiiden, 'role': analystiden}
-                async with sess.post(f'https://localhost:{port}/api/v1/auth/revoke', json=info) as resp:
-                    item = await resp.json()
-                    self.eq('ok', item.get('status'))
-                    roles = item['result']['roles']
-                    self.len(0, roles)
 
             # lets try out session based login
-            core.insecure = False
 
             async with self.getHttpSess() as sess:
 
@@ -449,33 +438,33 @@ class HttpApiTest(s_tests.SynTest):
 
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/adduser', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/addrole', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/grant', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/revoke', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/user/{visiiden}', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/role/{analystiden}', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 async with sess.get(f'https://localhost:{port}/api/v1/storm/nodes', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
-                rules = [(True, ('node:add',))]
+                rules = [(True, ('node', 'add',))]
                 info = {'name': 'derpuser', 'passwd': 'derpuser', 'rules': rules}
                 async with sess.post(f'https://localhost:{port}/api/v1/auth/adduser', json=info) as resp:
                     retn = await resp.json()
@@ -505,9 +494,10 @@ class HttpApiTest(s_tests.SynTest):
 
                 async with sess.post(f'https://localhost:{port}/api/v1/login', data=b'asdf') as resp:
                     retn = await resp.json()
-                    self.eq('BadJson', retn.get('code'))
+                    self.eq('SchemaViolation', retn.get('code'))
 
-                async with sess.post(f'https://localhost:{port}/api/v1/login', json={'user': 'derpuser', 'passwd': 'derpuser'}) as resp:
+                async with sess.post(f'https://localhost:{port}/api/v1/login',
+                                     json={'user': 'derpuser', 'passwd': 'derpuser'}) as resp:
                     retn = await resp.json()
                     self.eq('ok', retn.get('status'))
                     self.eq('derpuser', retn['result']['name'])
@@ -538,11 +528,56 @@ class HttpApiTest(s_tests.SynTest):
                     retn = await resp.json()
                     self.eq('AuthDeny', retn.get('code'))
 
-    async def test_http_model(self):
+    async def test_http_impersonate(self):
 
         async with self.getTestCore() as core:
 
-            core.insecure = False
+            host, port = await core.addHttpsPort(0, host='127.0.0.1')
+
+            visi = await core.auth.addUser('visi')
+
+            await visi.setPasswd('secret')
+            await visi.addRule((True, ('impersonate',)))
+
+            opts = {'user': core.auth.rootuser.iden}
+
+            async with self.getHttpSess() as sess:
+
+                async with sess.post(f'https://localhost:{port}/api/v1/login', json={'user': 'visi', 'passwd': 'secret'}) as resp:
+                    retn = await resp.json()
+                    self.eq('ok', retn.get('status'))
+
+                data = {'query': '[ inet:ipv4=1.2.3.4 ]', 'opts': opts}
+
+                podes = []
+                async with sess.get(f'https://localhost:{port}/api/v1/storm/nodes', json=data) as resp:
+
+                    async for byts, x in resp.content.iter_chunks():
+
+                        if not byts:
+                            break
+
+                        podes.append(json.loads(byts))
+
+                self.eq(podes[0][0], ('inet:ipv4', 0x01020304))
+
+                msgs = []
+                data = {'query': '[ inet:ipv4=5.5.5.5 ]', 'opts': opts}
+
+                async with sess.get(f'https://localhost:{port}/api/v1/storm', json=data) as resp:
+
+                    async for byts, x in resp.content.iter_chunks():
+
+                        if not byts:
+                            break
+
+                        msgs.append(json.loads(byts))
+                podes = [m[1] for m in msgs if m[0] == 'node']
+                self.eq(podes[0][0], ('inet:ipv4', 0x05050505))
+
+    async def test_http_model(self):
+
+        async with self.getTestCore() as core:
 
             visi = await core.auth.addUser('visi')
 
@@ -555,7 +590,8 @@ class HttpApiTest(s_tests.SynTest):
 
                 self.len(0, core.sessions)  # zero sessions..
 
-                async with sess.post(f'https://localhost:{port}/api/v1/login', json={'user': 'visi', 'passwd': 'secret'}) as resp:
+                async with sess.post(f'https://localhost:{port}/api/v1/login',
+                                     json={'user': 'visi', 'passwd': 'secret'}) as resp:
                     retn = await resp.json()
                     self.eq('ok', retn.get('status'))
                     self.eq('visi', retn['result']['name'])
@@ -659,8 +695,6 @@ class HttpApiTest(s_tests.SynTest):
 
         async with self.getTestCore() as core:
 
-            core.insecure = False
-
             visi = await core.auth.addUser('visi')
 
             await visi.setAdmin(True)
@@ -677,7 +711,7 @@ class HttpApiTest(s_tests.SynTest):
 
                 async with sess.get(f'https://localhost:{port}/api/v1/storm', data=b'asdf') as resp:
                     item = await resp.json()
-                    self.eq('BadJson', item.get('code'))
+                    self.eq('SchemaViolation', item.get('code'))
 
                 node = None
                 body = {'query': '[ inet:ipv4=1.2.3.4 ]'}
@@ -694,6 +728,7 @@ class HttpApiTest(s_tests.SynTest):
                         if mesg[0] == 'node':
                             node = mesg[1]
 
+                    self.nn(node)
                     self.eq(0x01020304, node[0][1])
 
                 async with sess.post(f'https://localhost:{port}/api/v1/storm', json=body) as resp:
@@ -740,7 +775,7 @@ class HttpApiTest(s_tests.SynTest):
             # Run http instead of https for this test
             host, port = await core.addHttpsPort(0, host='127.0.0.1')
 
-            root = core.auth.getUserByName('root')
+            root = await core.auth.getUserByName('root')
             await root.setPasswd('secret')
 
             url = f'https://localhost:{port}/api/v1/healthcheck'
