@@ -2,11 +2,35 @@ import os
 import json
 
 import synapse.common as s_common
-import synapse.lib.msgpack as s_msgpack
+
+import synapse.lib.cell as s_cell
 import synapse.lib.jupyter as s_jupyter
+import synapse.lib.msgpack as s_msgpack
+import synapse.lib.stormsvc as s_stormsvc
 
 import synapse.tests.utils as s_t_utils
 
+class TstsvcApi(s_cell.CellApi, s_stormsvc.StormSvc):
+    _storm_svc_name = 'testsvc'
+    _storm_svc_pkgs = (
+        {
+            'name': 'testsvc',
+            'version': (0, 0, 1),
+            'commands': (
+                {
+                    'name': 'testsvc.magic',
+                    'storm': '[inet:ipv4=0]',
+                },
+            )
+        },
+    )
+
+    async def testmeth(self):
+        return 'shazam'
+
+class Tstsvc(s_cell.Cell):
+    cellapi = TstsvcApi
+
 class JupyterTest(s_t_utils.SynTest):
     testmods = ['synapse.tests.utils.TestModule']
 
@@ -30,6 +54,26 @@ class JupyterTest(s_t_utils.SynTest):
         await prox.fini()
         self.true(prox.isfini)
 
+    async def test_tempcorecmdrstormsvc(self):
+        cmdrcore, svcprox = await s_jupyter.getTempCoreCmdrStormsvc('testsvc', Tstsvc.anit)
+
+        self.false(cmdrcore.isfini)
+        self.false(svcprox.isfini)
+
+        mesgs = await cmdrcore.storm('service.list')
+        self.true(any(['True (testsvc)' in str(mesg) for mesg in mesgs]))
+
+        nodes = await cmdrcore.eval('testsvc.magic')
+        self.len(1, nodes)
+
+        self.eq('shazam', await svcprox.testmeth())
+
+        await cmdrcore.fini()
+        self.true(cmdrcore.isfini)
+
+        await svcprox.fini()
+        self.true(svcprox.isfini)
+
     async def test_cmdrcore(self):
 
         async with self.getTestCoreAndProxy() as (realcore, core):
@@ -75,9 +119,8 @@ class JupyterTest(s_t_utils.SynTest):
                 data = [
                     (('test:int', 137), {}),
                 ]
-                guid = s_common.guid()
-                ret = await cmdrcore.addFeedData('syn.nodes', data, (guid, 1))
-                self.eq(ret, 2)
+
+                await cmdrcore.addFeedData('syn.nodes', data)
                 podes = await cmdrcore.eval('test:int=137',
                                             num=1, cmdr=False)
                 self.len(1, podes)
