@@ -8,6 +8,8 @@ import synapse.tests.utils as s_test
 import synapse.lib.cell as s_cell
 import synapse.lib.stormsvc as s_stormsvc
 
+import synapse.tools.backup as s_tools_backup
+
 old_pkg = {
     'name': 'old',
     'version': (0, 0, 1),
@@ -80,25 +82,28 @@ new_pkg = {
 class OldServiceAPI(s_cell.CellApi, s_stormsvc.StormSvc):
     _storm_svc_name = 'chng'
     _storm_svc_pkgs = (
-        old_pkg,
+        old_pkg,  # type: ignore
     )
 
 class NewServiceAPI(s_cell.CellApi, s_stormsvc.StormSvc):
     _storm_svc_name = 'chng'
     _storm_svc_pkgs = (
-        new_old_pkg,
+        new_old_pkg,  # type: ignore
         new_pkg,
     )
 
 class ChangingService(s_cell.Cell):
-    confdefs = (
-        ('updated', {'type': 'bool', 'defval': False,
-                     'doc': 'If true, serve new cell api'}),
-    )
+    confdefs = {
+        'updated': {
+            'type': 'boolean',
+            'default': False,
+            'description': 'If true, serve new cell api.',
+        }
+    }
 
     async def getTeleApi(self, link, mesg, path):
 
-        user = self._getCellUser(mesg)
+        user = await self._getCellUser(mesg)
 
         if self.conf.get('updated'):
             return await NewServiceAPI.anit(self, link, user)
@@ -108,7 +113,7 @@ class ChangingService(s_cell.Cell):
 class RealService(s_stormsvc.StormSvc):
     _storm_svc_name = 'real'
     _storm_svc_pkgs = (
-        {
+        {  # type: ignore
             'name': 'foo',
             'version': (0, 0, 1),
             'modules': (
@@ -132,6 +137,10 @@ class RealService(s_stormsvc.StormSvc):
                     ),
                     'storm': '[ inet:ipv4=1.2.3.4 :asn=$lib.service.get($cmdconf.svciden).asn() ]',
                 },
+                {
+                    'name': 'yoyo',
+                    'storm': 'for $ipv4 in $lib.service.get($cmdconf.svciden).ipv4s() { [inet:ipv4=$ipv4] }',
+                },
             )
         },
     )
@@ -153,10 +162,27 @@ class RealService(s_stormsvc.StormSvc):
         yield '5.5.5.5'
         yield '123.123.123.123'
 
+class NodeCreateService(s_stormsvc.StormSvc):
+    _storm_svc_name = 'ncreate'
+    _storm_svc_pkgs = (
+        {
+            'name': 'ncreate',
+            'version': (0, 0, 1),
+            'commands': (
+                {
+                    'name': 'baz',
+                    'storm': '''
+                    [inet:ipv4=8.8.8.8]
+                    ''',
+                },
+            )
+        },
+    )
+
 class BoomService(s_stormsvc.StormSvc):
     _storm_svc_name = 'boom'
     _storm_svc_pkgs = (
-        {
+        {  # type: ignore
             'name': 'boom',
             'version': (0, 0, 1),
             'modules': (
@@ -186,7 +212,7 @@ class BoomService(s_stormsvc.StormSvc):
 class DeadService(s_stormsvc.StormSvc):
     _storm_svc_name = 'dead'
     _storm_svc_pkgs = (
-        {
+        {  # type: ignore
             'name': 'dead',
             'version': (0, 0, 1),
             'commands': (
@@ -213,13 +239,13 @@ class NoService:
 class LifterService(s_stormsvc.StormSvc):
     _storm_svc_name = 'lifter'
     _storm_svc_pkgs = (
-        {
+        {  # type: ignore
             'name': 'lifter',
             'version': (0, 0, 1),
             'commands': (
                 {
                     'name': 'lifter',
-                    'desc': 'Lift inet:ipv4=1.2.3.4',
+                    'descr': 'Lift inet:ipv4=1.2.3.4',
                     'storm': 'inet:ipv4=1.2.3.4',
                 },
             ),
@@ -234,23 +260,29 @@ class LifterService(s_stormsvc.StormSvc):
         },
     }
 
-class StormvarService(s_stormsvc.StormSvc):
+class StormvarService(s_cell.CellApi, s_stormsvc.StormSvc):
     _storm_svc_name = 'stormvar'
     _storm_svc_pkgs = (
-        {
+        {  # type: ignore
             'name': 'stormvar',
             'version': (0, 0, 1),
             'commands': (
                 {
                     'name': 'magic',
+                    'descr': 'Test stormvar support.',
                     'cmdargs': (
-                        ('--stormvar', {'default': False, 'action': 'store_true'}),
                         ('name', {}),
+                        ('--debug', {'default': False, 'action': 'store_true'})
                     ),
+                    'forms': {
+                        'input': ('test:str', 'test:int'),
+                        'output': ('test:comp', 'inet:ipv4'),
+                    },
                     'storm': '''
-                    $name = $cmdopts.name
-                    $fooz = $lib.vars.get($name)
-                    $fooz = $path.vars.$name
+                    $fooz = $cmdopts.name
+                    if $cmdopts.debug {
+                        $lib.print('DEBUG: fooz={fooz}', fooz=$fooz)
+                    }
                     $lib.print('my foo var is {f}', f=$fooz)
                     ''',
                 },
@@ -258,6 +290,10 @@ class StormvarService(s_stormsvc.StormSvc):
         },
     )
 
+class StormvarServiceCell(s_cell.Cell):
+
+    cellapi = StormvarService
+
 @contextlib.contextmanager
 def patchcore(core, attr, newfunc):
     origvalu = getattr(core, attr)
@@ -273,24 +309,24 @@ class StormSvcTest(s_test.SynTest):
 
         async with self.getTestCore() as core:
 
-            msgs = await core.streamstorm('service.add --help').list()
+            msgs = await core.stormlist('service.add --help')
             self.stormIsInPrint(f'Add a storm service to the cortex.', msgs)
 
-            msgs = await core.streamstorm('service.del --help').list()
+            msgs = await core.stormlist('service.del --help')
             self.stormIsInPrint(f'Remove a storm service from the cortex.', msgs)
 
-            msgs = await core.streamstorm('service.list --help').list()
+            msgs = await core.stormlist('service.list --help')
             self.stormIsInPrint(f'List the storm services configured in the cortex.', msgs)
 
-            msgs = await core.streamstorm('service.add fake tcp://localhost:3333/foo').list()
+            msgs = await core.stormlist('service.add fake tcp://localhost:3333/foo')
             iden = core.getStormSvcs()[0].iden
             self.stormIsInPrint(f'added {iden} (fake): tcp://localhost:3333/foo', msgs)
 
-            msgs = await core.streamstorm('service.list').list()
+            msgs = await core.stormlist('service.list')
             self.stormIsInPrint('Storm service list (iden, ready, name, url):', msgs)
             self.stormIsInPrint(f'    {iden} False (fake): tcp://localhost:3333/foo', msgs)
 
-            msgs = await core.streamstorm(f'service.del {iden[:4]}').list()
+            msgs = await core.stormlist(f'service.del {iden[:4]}')
             self.stormIsInPrint(f'removed {iden} (fake): tcp://localhost:3333/foo', msgs)
 
     async def test_storm_svcs_bads(self):
@@ -316,9 +352,9 @@ class StormSvcTest(s_test.SynTest):
                 await core.nodes(f'service.add fake {lurl}')
                 await core.nodes('$lib.service.wait(fake)')
 
-                core.svcsbyname['fake'].client._t_conf['timeout'] = 0.1
+                core.svcsbyname['fake'].proxy._t_conf['timeout'] = 0.1
 
-            await core.svcsbyname['fake'].client._t_proxy.waitfini(6)
+            await core.svcsbyname['fake'].proxy._t_proxy.waitfini(6)
 
             with self.raises(s_exc.StormRuntimeError):
                 await core.nodes('[ inet:ipv4=6.6.6.6 ] | ohhai')
@@ -340,7 +376,7 @@ class StormSvcTest(s_test.SynTest):
 
             await core.setStormCmd(cdef)
 
-            nodes = await core.nodes('[ test:str=asdf ] | lulz')
+            await core.nodes('[ test:str=asdf ] | lulz')
 
     async def test_storm_pkg_persist(self):
 
@@ -363,6 +399,37 @@ class StormSvcTest(s_test.SynTest):
                 nodes = await core.nodes('foobar')
                 self.eq(nodes[0].ndef, ('inet:asn', 30))
 
+    async def test_storm_svc_nodecreate(self):
+        '''
+        Regression test for var leakage
+        '''
+        with self.getTestDir() as dirn:
+
+            async with self.getTestDmon() as dmon:
+
+                dmon.share('real', RealService())
+                dmon.share('ncreate', NodeCreateService())
+
+                host, port = dmon.addr
+
+                lurl = f'tcp://127.0.0.1:{port}/real'
+                murl = f'tcp://127.0.0.1:{port}/ncreate'
+
+                async with await s_cortex.Cortex.anit(dirn) as core:
+
+                    await core.nodes(f'service.add real {lurl}')
+                    await core.nodes(f'service.add ncreate {murl}')
+
+                    await core.nodes('$lib.service.wait(real)')
+                    await core.nodes('$lib.service.wait(ncreate)')
+
+                    await core.nodes('[inet:ipv4=1.2.3.3]')
+
+                    # baz yields inbound *and* a new node
+                    # yoyo calls cmdconf.svciden in an iterator
+                    nodes = await core.nodes('inet:ipv4=1.2.3.3 | baz | yoyo')
+                    self.len(5, {n.ndef for n in nodes})
+
     async def test_storm_svcs(self):
 
         with self.getTestDir() as dirn:
@@ -413,7 +480,7 @@ class StormSvcTest(s_test.SynTest):
                     await core.nodes('$lib.service.wait(lift)')
 
                     # check that new commands are displayed properly in help
-                    msgs = await core.streamstorm('help').list()
+                    msgs = await core.stormlist('help')
                     self.stormIsInPrint('service: fake', msgs)
                     self.stormIsInPrint('package: foo', msgs)
                     self.stormIsInPrint('foobar', msgs)
@@ -471,7 +538,7 @@ class StormSvcTest(s_test.SynTest):
 
                     # reach in and close the proxies
                     for ssvc in core.getStormSvcs():
-                        await ssvc.client._t_proxy.fini()
+                        await ssvc.proxy._t_proxy.fini()
 
                     nodes = await core.nodes('[ inet:ipv4=6.6.6.6 ] | ohhai')
                     self.len(2, nodes)
@@ -492,7 +559,7 @@ class StormSvcTest(s_test.SynTest):
 
                     # specifically call teardown
                     for svc in core.getStormSvcs():
-                        mesgs = await s_test.alist(core.streamstorm(f'service.del {svc.iden}'))
+                        mesgs = await core.stormlist(f'service.del {svc.iden}')
                         mesgs = [m[1].get('mesg') for m in mesgs if m[0] == 'print']
                         self.len(1, mesgs)
                         self.isin(f'removed {svc.iden} ({svc.name})', mesgs[0])
@@ -505,6 +572,7 @@ class StormSvcTest(s_test.SynTest):
                     self.eq(ans, reprs)
 
                     badiden = []
+
                     async def badSetStormSvcEvents(iden, evts):
                         badiden.append(iden)
                         raise s_exc.SynErr('Kaboom')
@@ -515,23 +583,23 @@ class StormSvcTest(s_test.SynTest):
                         'url': durl,
                     }
                     with patchcore(core, 'setStormSvcEvents', badSetStormSvcEvents):
-                        ssvc = await core.addStormSvc(sdef)
-                        await ssvc.ready.wait()
-                        await core.delStormSvc(ssvc.iden)
+                        svci = await core.addStormSvc(sdef)
+                        self.true(await core.waitStormSvc('dead', timeout=0.2))
+                        await core.delStormSvc(svci.get('iden'))
 
                     self.len(1, badiden)
-                    self.eq(ssvc.iden, badiden.pop())
+                    self.eq(svci.get('iden'), badiden.pop())
 
                     async def badRunStormSvcAdd(iden):
                         badiden.append(iden)
                         raise s_exc.SynErr('Kaboom')
 
                     with patchcore(core, '_runStormSvcAdd', badRunStormSvcAdd):
-                        ssvc = await core.addStormSvc(sdef)
-                        await ssvc.ready.wait()
-                        await core.delStormSvc(ssvc.iden)
+                        svci = await core.addStormSvc(sdef)
+                        self.true(await core.waitStormSvc('dead', timeout=0.2))
+                        await core.delStormSvc(svci.get('iden'))
                     self.len(1, badiden)
-                    self.eq(ssvc.iden, badiden[0])
+                    self.eq(svci.get('iden'), badiden[0])
 
     async def test_storm_svc_restarts(self):
 
@@ -541,7 +609,7 @@ class StormSvcTest(s_test.SynTest):
                     async with await ChangingService.anit(svcd) as chng:
                         chng.dmon.share('chng', chng)
 
-                        root = chng.auth.getUserByName('root')
+                        root = await chng.auth.getUserByName('root')
                         await root.setPasswd('root')
 
                         info = await chng.dmon.listen('tcp://127.0.0.1:0/')
@@ -569,7 +637,7 @@ class StormSvcTest(s_test.SynTest):
                     self.true(await waiter.wait(10))
                     async with await ChangingService.anit(svcd, {'updated': True}) as chng:
                         chng.dmon.share('chng', chng)
-                        _ = await chng.dmon.listen(f'tcp://127.0.0.1:{port}/')
+                        await chng.dmon.listen(f'tcp://127.0.0.1:{port}/')
 
                         await core.nodes('$lib.service.wait(chng)')
 
@@ -588,47 +656,134 @@ class StormSvcTest(s_test.SynTest):
                         pkg = await core.getStormPkg('old')
                         self.eq(pkg.get('version'), (0, 1, 0))
 
-                    cdef = OldServiceAPI._storm_svc_pkgs[0].get('commands')[-1]
-                    cdef['cmdconf'] = {'svciden': 'fakeiden'}
-                    await core.setStormCmd(cdef)
-                    self.nn(core.getStormCmd('oldcmd'))
+            # This test verifies that storm commands loaded from a previously connected service are still available,
+            # even if the service is not available now
+            with self.getLoggerStream('synapse.lib.nexus') as stream:
+                async with await s_cortex.Cortex.anit(dirn) as core:
+                    self.nn(core.getStormCmd('newcmd'))
+                    self.nn(core.getStormCmd('new.baz'))
+                    self.nn(core.getStormCmd('old.bar'))
+                    self.nn(core.getStormCmd('runtecho'))
+                    self.none(core.getStormCmd('oldcmd'))
+                    self.none(core.getStormCmd('old.baz'))
+                    self.isin('old', core.stormpkgs)
+                    self.isin('new', core.stormpkgs)
+                    self.isin('echo', core.stormmods)
+                    self.isin('old.bar', core.stormmods)
+                    self.isin('new.baz', core.stormmods)
+                    self.notin('old.baz', core.stormmods)
 
-            async with await s_cortex.Cortex.anit(dirn) as core:
-                self.nn(core.getStormCmd('newcmd'))
-                self.nn(core.getStormCmd('new.baz'))
-                self.nn(core.getStormCmd('old.bar'))
-                self.nn(core.getStormCmd('runtecho'))
-                self.none(core.getStormCmd('oldcmd'))
-                self.none(core.getStormCmd('old.baz'))
-                self.isin('old', core.stormpkgs)
-                self.isin('new', core.stormpkgs)
-                self.isin('echo', core.stormmods)
-                self.isin('old.bar', core.stormmods)
-                self.isin('new.baz', core.stormmods)
-                self.notin('old.baz', core.stormmods)
+            stream.seek(0)
+            mesgs = stream.read()
+            self.notin('Exception while replaying', mesgs)
 
     async def test_storm_vars(self):
+
+        async with self.getTestCoreProxSvc(StormvarServiceCell) as (core, prox, svc):
+
+            await core.nodes('[ inet:ipv4=1.2.3.4 inet:ipv4=5.6.7.8 ]')
+
+            scmd = f'inet:ipv4=1.2.3.4 $foo=$node.repr() | magic $foo'
+            msgs = await core.stormlist(scmd)
+            self.stormIsInPrint('my foo var is 1.2.3.4', msgs)
+
+            scmd = f'inet:ipv4=1.2.3.4 inet:ipv4=5.6.7.8 $foo=$node.repr() | magic $foo'
+            msgs = await core.stormlist(scmd)
+            self.stormIsInPrint('my foo var is 1.2.3.4', msgs)
+            self.stormIsInPrint('my foo var is 5.6.7.8', msgs)
+
+            scmd = f'$foo=8.8.8.8 | magic $foo'
+            msgs = await core.stormlist(scmd)
+            self.stormIsInPrint('my foo var is 8.8.8.8', msgs)
+
+            scmd = f'$foo=8.8.8.8 | magic $foo --debug'
+            msgs = await core.stormlist(scmd)
+            self.stormIsInPrint('DEBUG: fooz=8.8.8.8', msgs)
+            self.stormIsInPrint('my foo var is 8.8.8.8', msgs)
+
+            scmd = f'$foo=8.8.8.8 | magic --debug $foo'
+            msgs = await core.stormlist(scmd)
+            self.stormIsInPrint('DEBUG: fooz=8.8.8.8', msgs)
+            self.stormIsInPrint('my foo var is 8.8.8.8', msgs)
+
+            scmd = 'inet:ipv4=1.2.3.4 inet:ipv4=5.6.7.8 $foo=$node.repr() | magic $foo --debug'
+            msgs = await core.stormlist(scmd)
+            self.stormIsInPrint('my foo var is 1.2.3.4', msgs)
+            self.stormIsInPrint('DEBUG: fooz=1.2.3.4', msgs)
+            self.stormIsInPrint('my foo var is 5.6.7.8', msgs)
+            self.stormIsInPrint('DEBUG: fooz=5.6.7.8', msgs)
+
+    async def test_storm_svc_mirror(self):
+
         with self.getTestDir() as dirn:
+
+            path00 = s_common.gendir(dirn, 'core00')
+            path01 = s_common.gendir(dirn, 'core01')
+
             async with self.getTestDmon() as dmon:
-                dmon.share('svar', StormvarService())
+
+                dmon.share('real', RealService())
                 host, port = dmon.addr
-                surl = f'tcp://127.0.0.1:{port}/svar'
+                lurl = f'tcp://127.0.0.1:{port}/real'
 
-                async with await s_cortex.Cortex.anit(dirn) as core:
-                    await core.nodes(f'service.add svar {surl}')
-                    await core.nodes('$lib.service.wait(svar)')
+                async with self.getTestCore(dirn=path00) as core00:
+                    await core00.nodes('[ inet:ipv4=1.2.3.4 ]')
 
-                    await core.nodes('[ inet:ipv4=1.2.3.4 inet:ipv4=5.6.7.8 ]')
+                s_tools_backup.backup(path00, path01)
 
-                    scmd = f'inet:ipv4=1.2.3.4 $foo=$node.repr() | magic --stormvar foo'
-                    msgs = await core.streamstorm(scmd).list()
-                    self.stormIsInPrint('my foo var is 1.2.3.4', msgs)
+                async with self.getTestCore(dirn=path00) as core00:
 
-                    scmd = f'inet:ipv4=1.2.3.4 inet:ipv4=5.6.7.8 $foo=$node.repr() | magic --stormvar foo'
-                    msgs = await core.streamstorm(scmd).list()
-                    self.stormIsInPrint('my foo var is 1.2.3.4', msgs)
-                    self.stormIsInPrint('my foo var is 5.6.7.8', msgs)
+                    self.false(core00.mirror)
 
-                    scmd = f'$foo="8.8.8.8" | magic --stormvar foo'
-                    msgs = await core.streamstorm(scmd).list()
-                    self.stormIsInPrint('my foo var is 8.8.8.8', msgs)
+                    url = core00.getLocalUrl()
+
+                    conf = {'mirror': url}
+                    async with await s_cortex.Cortex.anit(dirn=path01, conf=conf) as core01:
+
+                        await core01.sync()
+
+                        self.true(core01.mirror)
+
+                        # Add a storm service
+                        await core01.nodes(f'service.add real {lurl}')
+                        await core01.nodes('$lib.service.wait(real)')
+
+                        # Make sure it shows up on leader
+                        msgs = await core00.stormlist('help')
+                        self.stormIsInPrint('service: real', msgs)
+                        self.stormIsInPrint('package: foo', msgs)
+                        self.stormIsInPrint('foobar', msgs)
+                        self.isin('foo.bar', core00.stormmods)
+
+                        queue = core00.multiqueue.list()
+                        self.len(1, queue)
+                        self.eq('vertex', queue[0]['name'])
+                        self.nn(core00.getStormCmd('ohhai'))
+
+                        # Make sure it shows up on mirror
+                        msgs = await core01.stormlist('help')
+                        self.stormIsInPrint('service: real', msgs)
+                        self.stormIsInPrint('package: foo', msgs)
+                        self.stormIsInPrint('foobar', msgs)
+                        self.isin('foo.bar', core01.stormmods)
+
+                        queue = core01.multiqueue.list()
+                        self.len(1, queue)
+                        self.eq('vertex', queue[0]['name'])
+                        self.nn(core01.getStormCmd('ohhai'))
+
+                        # Delete storm service
+                        iden = core01.getStormSvcs()[0].iden
+                        await core01.delStormSvc(iden)
+                        await core01.sync()
+
+                        # Make sure it got removed from both
+                        self.none(core00.getStormCmd('ohhai'))
+                        queue = core00.multiqueue.list()
+                        self.len(0, queue)
+                        self.notin('foo.bar', core00.stormmods)
+
+                        self.none(core01.getStormCmd('ohhai'))
+                        queue = core01.multiqueue.list()
+                        self.len(0, queue)
+                        self.notin('foo.bar', core01.stormmods)
