@@ -1,3 +1,4 @@
+import synapse.exc as s_exc
 
 import synapse.lib.time as s_time
 
@@ -30,3 +31,6 @@ class TimeTest(s_t_utils.SynTest):
         self.eq(s_time.parse('205012170304'), 2554859040000)
         self.eq(s_time.parse('20501217030432'), 2554859072000)
         self.eq(s_time.parse('20501217030432101'), 2554859072101)
+        self.eq(s_time.parse('205012170304321015'), 2554859072101)
+        self.eq(s_time.parse('20501217030432101567'), 2554859072101)
+        self.raises(s_exc.BadTypeValu, s_time.parse, '2050121703043210156789')
