@@ -54,6 +54,7 @@ class Foo:
 
     def __init__(self):
         self.sleepg_evt = asyncio.Event()
+        self.simplesleep_evt = asyncio.Event()
 
     def bar(self, x, y):
         return x + y
@@ -77,6 +78,11 @@ class Foo:
             raise
         yield ('fini', {})
 
+    async def simplesleep(self):
+        self.simplesleep_evt.set()
+        await asyncio.sleep(10)
+        return 42
+
     def genr(self):
         yield 10
         yield 20
@@ -161,7 +167,7 @@ class TeleAuth(s_telepath.Aware):
         return x + y
 
 class ConsulV1Handler(s_httpapi.Handler):
-    '''Fake Hasicorp Consul /v1/catalog/service/:service: handler'''
+    '''Fake Hashicorp Consul /v1/catalog/service/:service: handler'''
     async def get(self, servicename):
         data = self.cell.consul_data
         if servicename != data[0].get('ServiceName'):
@@ -828,7 +834,7 @@ class TeleTest(s_t_utils.SynTest):
             # link not being placed back into the pool
             self.eq(await genr0.list(), (10, 20, 30))
             self.len(2, prox.links)
-            links = set(l for l in prox.links)
+            links = set(lnk for lnk in prox.links)
             self.notin(l0, links)
             # And that link l0 has been fini'd
             self.true(l0.isfini)
@@ -884,6 +890,26 @@ class TeleTest(s_t_utils.SynTest):
                 # Ensure we logged the cancellation.
                 self.true(await stream.wait(6))
 
+    async def test_link_fini_breaking_tasks2(self):
+        '''
+        Similar to the previous test, except tears down a proxy that another task is using
+        '''
+        foo = Foo()
+        async with self.getTestDmon() as dmon:
+            dmon.share('foo', foo)
+            url = f'tcp://127.0.0.1:{dmon.addr[1]}/foo'
+            prox = await s_telepath.openurl(url)  # type: Foo
+
+            async def doit():
+                retn = await prox.simplesleep()
+                return retn
+
+            task = dmon.schedCoro(doit())
+            self.true(await s_coro.event_wait(foo.simplesleep_evt, 2))
+            await prox.fini()
+
+            await self.asyncraises(s_exc.LinkShutDown, task)
+
     async def test_discovery_consul(self):
 
         # Example data from https://www.consul.io/api/catalog.html
@@ -940,7 +966,7 @@ class TeleTest(s_t_utils.SynTest):
         ]
         with self.getTestDir() as dirn:
             async with await s_cell.Cell.anit(dirn) as cell:
-                root = cell.auth.getUserByName('root')
+                root = await cell.auth.getUserByName('root')
                 await root.setPasswd('root')
                 cell.consul_data = consul_data
                 hhost, hport = await cell.addHttpsPort(0)
